/*
	$Id$

	Copyright (C) 2015 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.formatter;

import java.util.List;

import net.sf.basedb.clients.web.Base;
import net.sf.basedb.clients.web.ModeInfo;
import net.sf.basedb.clients.web.extensions.fileviewer.FileViewerUtil;
import net.sf.basedb.clients.web.taglib.Page;
import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.File;
import net.sf.basedb.core.FileSet;
import net.sf.basedb.core.FileSetMember;
import net.sf.basedb.core.Include;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.PermissionDeniedException;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.util.formatter.Formatter;

/**
	Formatter implementation for file sets that lists all member files 
	of a given file type
	@author nicklas
	@since 3.5
*/
public class DataFileFormatter 
	implements Formatter<FileSet>
{

	private final DbControl dc;
	private final DataFileType fileType;
	private final ModeInfo mode;
	private final FileViewerUtil fileViewer;
	private ItemQuery<FileSetMember> memberQuery;
	
	/**
		Creates a new formatter instance for the given file type and list mode.
	*/
	public DataFileFormatter(DbControl dc, DataFileType fileType, ModeInfo mode)
	{
		this(dc, fileType, mode, null);
	}
	
	/**
		Creates a new formatter instance for the given file type and list mode.
		@since 3.17
	*/
	public DataFileFormatter(DbControl dc, DataFileType fileType, ModeInfo mode, FileViewerUtil fileViewer)
	{
		this.dc = dc;
		this.fileType = fileType;
		this.mode = mode;
		this.fileViewer = fileViewer;
	}

	/**
		Lists the name of all member files in the file set that are of the
		configured file type.
	*/
	@Override
	public String format(FileSet value) 
	{
		if (memberQuery == null)
		{
			memberQuery = FileSetMember.getQuery(null);
			memberQuery.restrict(
				Restrictions.eq(
					Hql.property("dataFileType"), 
					Hql.entity(fileType)
				));
			memberQuery.setIncludes(Include.ALL);
		}
		memberQuery.setEntityParameter("fileSet", value);
		
		List<FileSetMember> members = memberQuery.list(dc);
		String ID = dc.getSessionControl().getId();
		StringBuilder sb = new StringBuilder();
		for (FileSetMember mbr : members)
		{
			File f = null;
			try
			{
				f = mbr.getFile();
			}
			catch (PermissionDeniedException ex)
			{}
			if (mode.hasPropertyLink())
			{
				if (fileViewer != null)
				{
					sb.append(Base.getLinkedName(ID, f, f == null, mode.hasEditLink())).append(" ");
					fileViewer.getFileContext().setCurrentDataFile((BasicItem)value.getItem(), f, fileType, mbr);
					sb.append(fileViewer.renderAsString());
				}
				else
				{
					sb.append(Base.getLinkedFile(ID, f, f == null, mode.hasEditLink(), true, Page.ROOT()));
				}
			}
			else
			{
				sb.append(Base.getEncodedName(f, f == null));
			}
			sb.append("<br>");
		}
		return sb.toString();
	}

	/**
		Not supported
	*/
	@Override
	public FileSet parseString(String value) 
	{
		throw new UnsupportedOperationException("parseString");
	}
	
}
