/*
	$Id: Upload.java 7990 2021-07-01 10:54:20Z nicklas $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.servlet;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.json.simple.JSONObject;

import net.sf.basedb.clients.web.Base;
import net.sf.basedb.clients.web.WebException;
import net.sf.basedb.clients.web.fileupload.FileUpload;
import net.sf.basedb.clients.web.fileupload.FileUploadProgress;
import net.sf.basedb.clients.web.fileupload.UploadAbortedException;
import net.sf.basedb.clients.web.fileupload.UploadedFile;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Directory;
import net.sf.basedb.core.File;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemContext;
import net.sf.basedb.core.ItemSubtype;
import net.sf.basedb.core.Operator;
import net.sf.basedb.core.Path;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.PropertyFilter;
import net.sf.basedb.core.SessionControl;
import net.sf.basedb.core.SimpleAbsoluteProgressReporter;
import net.sf.basedb.core.SystemItems;
import net.sf.basedb.core.Type;
import net.sf.basedb.util.FileUtil;
import net.sf.basedb.util.InputStreamSplitter;
import net.sf.basedb.util.Values;
import net.sf.basedb.util.error.ThrowableUtil;
import net.sf.basedb.util.zip.FileUnpacker;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;


/**
	The upload servlet is used to handle single file uploads. It has support
	for on-the-fly unpacking of compressed archives.
	 
	@author nicklas
	@since 2.16
	@base.modified $Date: 2021-07-01 12:54:20 +0200 (Thu, 01 Jul 2021) $
*/
public final class Upload
	extends HttpServlet
{


	private static final long serialVersionUID = 1404104707477507003L;

	@Override
	public void doPost(HttpServletRequest request, HttpServletResponse response)
		throws IOException, ServletException
	{

		SessionControl sc = null;
		FileUpload upload = null;
		FileUnpacker unpacker = null;
		DbControl dc = null;
		JSONObject json = new JSONObject();
		response.setContentType("application/json");
		response.setCharacterEncoding("UTF-8");
		response.setHeader("Cache-Control", "no-cache, max-age=0");
		json.put("status", "ok");

		try
		{
			// Get context information
			sc = Base.getExistingSessionControl(request, Permission.CREATE, Item.FILE);
			final String ID = sc.getId();
			final ItemContext cc = Base.getAndSetCurrentContext(sc, Item.FILE, null, null);
			final String root = request.getContextPath()+"/";
			final int maxTransferRate = Values.getInt(sc.getClientDefaultSetting("server.transferRate"), 0);
			final int maxRecent = Base.getMaxRecent(sc);
			final int directoryId = Values.getInt(request.getParameter("directory_id"), 
					SystemItems.getId(Directory.ROOT));
			File currentFile = (File)cc.getObject("item");

			// Prepare the upload and progress reporter
			upload = new FileUpload(request, maxTransferRate, FileUpload.DEFAULT_TIMEOUT);
			final FileUploadProgress progress = upload.getProgress();
			sc.setSessionSetting("FileUploadProgress", progress);
			
			// Parse to the first file section
			UploadedFile uploadedFile = upload.next();
			
			// Check that the user is allowed to create files in the given directory
			dc = sc.newDbControl(":File upload");
			final String subfolder = Values.getStringOrNull(upload.getParameter("subfolder"));
			Directory directory = Directory.getById(dc, directoryId);
			if (subfolder != null)
			{
				Path subFolderPath = new Path(directory.getPath().toString()+"/"+subfolder, Path.Type.DIRECTORY);
				Directory subdir = Directory.getNew(dc, subFolderPath);
				Directory checkRemoved = subdir;
				while (true)
				{
					// Reset 'removed' flag on subdirectories that we upload to
					if (checkRemoved.isRemoved() && checkRemoved.hasPermission(Permission.WRITE)) 
					{
						checkRemoved.setRemoved(false);
					}
					checkRemoved = checkRemoved.getParent();
					if (checkRemoved.equals(directory)) break;
				}
				directory = subdir;
			}
			directory.checkPermission(Permission.USE);
			
			// Get all form parameters
			boolean autoSelectFile = Values.getBoolean(upload.getParameter("autoSelect"), true);
			boolean checkMd5 = Values.getBoolean(upload.getParameter("check_md5"), false);
			boolean writeProtected = Values.getBoolean(upload.getParameter("write_protected"), false);
			boolean replaceExisting = Values.getBoolean(upload.getParameter("replace_existing"), false);
			Boolean compress = null; // null == auto by MIME type
			if (Values.getStringOrNull(upload.getParameter("compressed")) != null) 
			{
				compress = Values.getBoolean(upload.getParameter("compressed"));
			}
			int subtypeId = Values.getInt(upload.getParameter("subtype_id"));
			String description = Values.getStringOrNull(upload.getParameter("description"));
			String characterSet = Values.getStringOrNull(upload.getParameter("characterSet"));
			int zipFormat = Values.getInt(upload.getParameter("zip_format"), 0);
			boolean isZipFile = zipFormat > 0;
			boolean unpackZip = isZipFile && Values.getBoolean(upload.getParameter("zip_unpack"), false);
			boolean overwriteZip = unpackZip && Values.getBoolean(upload.getParameter("zip_overwrite"), false);
			boolean keepZip = unpackZip && Values.getBoolean(upload.getParameter("zip_keep"), true);
			int numUnpackedFiles = 0;

			
			if (uploadedFile != null)
			{
				File f = null;
				OutputStream toFile = null;

				// Set file properties, check for duplicate files, save new file etc.
				if (!isZipFile || !unpackZip || keepZip)
				{
					boolean fileExists = false;
					boolean stopIfFileExists = true;

					// Are we currently working with an existing file?
					if (currentFile == null)
					{	
						// No... it's a new upload... try to create a new file
						f = File.getFile(dc, directory, uploadedFile.getFilename(), true);
						if (f.isInDatabase())
						{
							// A different file already exists... should we overwrite it?
							fileExists = true;
							if (f.isRemoved() || (isZipFile && overwriteZip) || replaceExisting)
							{
								stopIfFileExists = false;
								f.setRemoved(false);
							}
						}
						else
						{
							// Save the (new) file
							dc.saveItem(f);
						}
						currentFile = f;
					}
					else
					{
						f = currentFile;
						// We are replacing an existing file... if the name has changed, check that is is unique
						if (!f.getName().equals(uploadedFile.getFilename()))
						{
							fileExists = File.exists(dc, directory, uploadedFile.getFilename());
							stopIfFileExists = true;
						}
						else 
						{
							stopIfFileExists = false;
							fileExists = true;
						}
						dc.reattachItem(f, false);
					}
					if (fileExists && stopIfFileExists)
					{
						throw new WebException("popup", "File already exists", "Another file with the same name already exists: {1}", uploadedFile.getFilename());
					}
				}
				
				if (f == null) 
				{
					// Create a dummy file that is not saved to the database
					// We use this so that options from the form can be sent 
					// to the unpacker.unpack() method
					f = File.getNew(dc, directory);
				}

				// Set various properties from the form on the file object
				f.setName(uploadedFile.getFilename());
				String serverMimeType = getServletContext().getMimeType(uploadedFile.getFilename());
				String browserMimeType = uploadedFile.getMimeType();
				f.setMimeTypeAuto(serverMimeType != null ? serverMimeType : browserMimeType, null);
				if (subtypeId >= 0) // < 0 = denied or unchanged
				{
					ItemSubtype subtype = subtypeId == 0 ? null : ItemSubtype.getById(dc, subtypeId);
					f.setItemSubtype(subtype);
					cc.setRecent(Item.ITEMSUBTYPE, subtype, maxRecent);
				}
				if (description != null) f.setDescription(description);
				if (characterSet != null) f.setCharacterSet(characterSet);
				if (!f.isDetached()) toFile = f.getUploadStream(checkMd5, compress);
				f.setWriteProtected(writeProtected);
				
				// Load plug-in for for unpacking (must implement FileUnpacker)
				if (isZipFile && unpackZip)
				{
					PluginDefinition plugin = PluginDefinition.getById(dc, zipFormat);
					unpacker = plugin.newInstance(FileUnpacker.class, sc, null, null);
				}
				
				// Read the uploaded data
				InputStream uploadStream = uploadedFile.getInputStream();
				if (unpacker == null)
				{
					// A regular file upload
					FileUtil.copy(uploadStream, toFile);
				}
				else
				{
					// We are uploading and unpacking a zip file on the fly
					InputStreamSplitter splitter = null;
					if (toFile != null) 
					{
						// The splitter will copy the original zip file (when keepZip==true)
						uploadStream = new InputStreamSplitter(uploadStream, true, true, toFile);
					}
					
					// A second progress reporter is used to track the unpacking
					long fileSize = progress.getTotalBytes() - progress.getTransferredBytes();
					SimpleAbsoluteProgressReporter unpackProgress = new SimpleAbsoluteProgressReporter(null, fileSize);
					sc.setSessionSetting("UnpackProgress", unpackProgress);
					numUnpackedFiles = unpacker.unpack(dc, directory, uploadStream, f, overwriteZip, unpackProgress);
				}
				uploadStream.close();

				// This will force the upload to continue reading the form data
				// to the end in case it contains info that we need
				upload.next();
			}
			
			// The upload could not be finished for some reason, for example: the user closed
			// the browser window or pulled the network cable, or...
			if (progress.getTransferredBytes() != progress.getTotalBytes())
			{
				throw new UploadAbortedException();
			}
			json.put("transferredBytes", progress.getTransferredBytes());
			json.put("timeInMillis", progress.getMillisecondsSinceStart());
			dc.commit();
			if (currentFile != null && autoSelectFile) 
			{
				// The new file should be selected automatically and if there is a filter we 
				// add a temporary filter on it's own row to make sure the new file is not filtered out
				cc.setId(currentFile.getId());
				if (cc.getNumPropertyFilters() > 0) 
				{
					int rows = cc.addFilterRows(1);
					PropertyFilter filter = new PropertyFilter("name", rows-1);
					filter.setOperator(Operator.EQ);
					filter.setValueType(Type.STRING);
					filter.setTemporary(true);
					filter.setValue(currentFile.getName());
					cc.setPropertyFilter(filter);
				}
			}
			if (unpackZip)
			{
				json.put("message", numUnpackedFiles + " file(s) unpacked successfully");
			}
			else
			{
				json.put("message", "File uploaded successfully");
			}
		}
		catch (Throwable t)
		{
			if (upload != null) upload.setException(t);
			json.clear();
			json.put("status", "error");
			json.put("message", t.getMessage());
			json.put("stacktrace", ThrowableUtil.stackTraceToString(t));
			if (t instanceof WebException)
			{
				WebException we = (WebException)t;
				json.put("title", we.getTitle());
				json.put("message", we.getDetail());
			}
			try
			{
				// We should wait here some time so that the progress updater
				// in the web GUI can see the exception 
				// (via sc.setSessionSetting("FileUploadProgress" in filemanager/upload/ajax.jsp)
				// and abort() the request from the client side as well
				// Otherwise, the browser will only re-try the file upload again, and again
				// See tickets: https://base.thep.lu.se/ticket/2018
				//              https://base.thep.lu.se/ticket/2019
				synchronized (t)
				{
					t.wait(10000);
				}
				// Just to let the progress updater have time to finish before we return
				Thread.sleep(500);
			}
			catch (InterruptedException ex)
			{}
		}
		finally
		{
			if (unpacker != null) unpacker.done();
			if (dc != null) dc.close();
			if (sc != null)
			{
				sc.setSessionSetting("FileUploadProgress", null);
				sc.setSessionSetting("UnpackProgress", null);
			}
			json.writeJSONString(response.getWriter());
		}
	}

	@Override
	protected void doGet(HttpServletRequest request, HttpServletResponse response)
		throws ServletException, IOException
	{
		response.sendRedirect("../../common/close_popup.jsp?refresh_opener=1");
	}
		
}
