/*
	$Id: Icon.java 7703 2019-04-11 07:31:26Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.taglib;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;

/**
	This tag is used to make it easier to add &lt;input&gt; elements
	on a BASE web page. It is mainly used for checkboxes and radio buttons.
	<p>
	<b>Example:</b>
	<pre>
   &lt;%@ taglib prefix="base" uri="/WEB-INF/base.tld" %&gt;
   &lt;base:input type="checkbox" name="item_id" value="55" checked="true" /&gt;
</pre>

	<p>
	<b>Syntax:</b>
	<pre>
   &lt;base:icon
      id=...
      clazz=...
      subclass=...
      style=...
      type=...
      name=...
      value=...
      checked=...
      visible=...
      tabindex=...
      title=...
   &gt;
</pre>

	<table class="as-grid">
	<caption>Tag attributes</caption>
	<tr>
		<td><b>Attribute</b></td>
		<td><b>Default value</b></td>
		<td><b>Required</b></td>
		<td><b>Description</b></td>
	</tr>
	<tr>
		<td>id</td>
		<td>-</td>
		<td>no</td>
		<td>The ID of the input element. The value of this attribute
		goes directly into the standard <code>id</code> attribute.
		This means that if you need a reference to the note object in your own
		JavaScript code, you may for example use the following code:
		<p>
		<code>
		icon = document.getElementById('&lt;id&gt;');
		</code>
		</td>
	</tr>
	<tr>
		<td>clazz</td>
		<td>-</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>class</code> attribute. This allows you to apply different styles
			to different inputs.
		</td>
	</tr>
	<tr>
		<td>subclass</td>
		<td>-</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>class</code> attribute. This allows you to apply different styles
			to different inputs.
		</td>
	</tr>
	<tr>
		<td>style</td>
		<td>-</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>style</code> attribute. This allows you to apply different styles
			to different inputs.
		</td>
	</tr>
	<tr>
		<td>type</td>
		<td></td>
		<td>yes</td>
		<td>
			The type of the input element.
		</td>
	</tr>
	<tr>
		<td>name</td>
		<td></td>
		<td>yes</td>
		<td>
			The name of the input element.
		</td>
	</tr>
	<tr>
		<td>value</td>
		<td></td>
		<td>no</td>
		<td>
			The value of the input element.
		</td>
	</tr>
	<tr>
		<td>checked</td>
		<td>false</td>
		<td>no</td>
		<td>
			If the input element should be checked or not.
		</td>
	</tr>
	<tr>
		<td>title</td>
		<td></td>
		<td>no</td>
		<td>
			A short explanation that will show up as a tooltip.
		</td>
	</tr>
	<tr>
		<td>visible</td>
		<td>true</td>
		<td>no</td>
		<td>
			If the input should be visible or not.
		</td>
	</tr>
	<tr>
		<td>tabindex</td>
		<td>0</td>
		<td>no</td>
		<td>
			Tab index value to control the focus order. Use -1 to disable tabbing to the input.
		</td>
	</tr>
	</table>

	@author Nicklas
	@since 3.18.1
*/
public class Input 
	extends StylableTag
{

	private static final long serialVersionUID = 8306637459722729777L;

	/**
		The type of input.
	*/
	private String type;
	
	/**
		The name of the input element. If not set,
		the ID is used.
	*/
	private String name;
	
	/**
		The value of the input element.
	*/
	private String value;
	
	/**
		TRUE if the input element is checked.
	*/
	private boolean checked;
	
	/**
		An optional title.
	*/
	private String title = null;
	
	/**
		If the input should be visible or not.
	*/
	private boolean visible = true;

	
	private int tabIndex;
	
	public Input()
	{
		super(null);
	}
	
	/*
		Taglib initialization methods
	*/
	public void setType(String type)
	{
		this.type = type;
	}
	public String getType()
	{
		return type;
	}

	public void setName(String name)
	{
		this.name = name;
	}
	public void setName(int name)
	{
		this.name = Integer.toString(name);
	}
	public String getName()
	{
		return name == null ? getId() : name;
	}

	public void setValue(String value)
	{
		this.value = value;
	}
	public void setValue(int value)
	{
		this.value = Integer.toString(value);
	}
	public String getValue()
	{
		return value;
	}
	
	public void setChecked(boolean checked)
	{
		this.checked = checked;
	}
	public boolean isChecked()
	{
		return checked;
	}

	public void setVisible(boolean visible)
	{
		this.visible = visible;
	}
	public boolean isVisible()
	{
		return visible;
	}
	

	public void setTitle(String title)
	{
		this.title = title;
	}
	public String getTitle()
	{
		return title;
	}

	public void setTabindex(int tabIndex)
	{
		this.tabIndex = tabIndex;
	}
	public int getTabindex()
	{
		return tabIndex;
	}

	@Override
	public int doStartTag()
		throws JspException
	{
		if (!isVisible()) return SKIP_BODY;

		StringBuilder sb = new StringBuilder();
		sb.append("<input ");
		addIdAndStyles(sb);
		if (getType() != null) sb.append(" type=\"").append(getType()).append("\"");
		if (getName() != null) sb.append(" name=\"").append(getName()).append("\"");
		if (getValue() != null) sb.append(" value=\"").append(getValue()).append("\"");
		if (getTitle() != null) sb.append(" title=\"").append(getTitle()).append("\"");
		if (isChecked()) sb.append(" checked");

		if (getTabindex() > 0) 
		{
			sb.append(" tabindex=\"").append(getTabindex()).append("\"");
		}

		addDynamicAttributes(sb);
		sb.append(">");
		try
		{
			pageContext.getOut().print(sb.toString());
		}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return SKIP_BODY;
	}
	
}

