/*
	$Id: Menuitem.java 7703 2019-04-11 07:31:26Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.taglib.menu;

import java.util.Iterator;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;

import net.sf.basedb.clients.web.extensions.DynamicActionAttribute;
import net.sf.basedb.clients.web.taglib.StylableTag;

/**
	This tag defines a menu item inside a drop-down menu. This tag can only be used
	inside a {@link Menu} tag, which should contain at least one of these tags.

	<p>
	<b>Syntax:</b>
	<pre>
   &lt;m:menuitem
      id=...
      clazz=...
      subclass=...
      style=...
      title=...
      url=...
      tooltip=...
      enabled=...
      icon=...
      visible=...
   &gt;
</pre>


	<table class="as-grid">
	<caption>Tag attributes</caption>
	<tr>
		<td><b>Attribute</b></td>
		<td><b>Default value</b></td>
		<td><b>Required</b></td>
		<td><b>Description</b></td>
	</tr>
	<tr>
		<td>id</td>
		<td>-</td>
		<td>no</td>
		<td>The ID of the menuitem. The value of this attribute
		goes directly into the standard HTML <code>id</code> attribute.
		This means that if you need a reference to the menuitem object in your own
		JavaScript code, you may for example use the following code:
		<p>
		<code>
		menuitem = document.getElementById('&lt;id&gt;');
		</code>
		</td>
	</tr>
	<tr>
		<td>style</td>
		<td>-</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>style</code> attribute. This allows you to apply different styles
			to different menu items.
		</td>
	</tr>
	<tr>
		<td>title</td>
		<td>-</td>
		<td>yes</td>
		<td>The text that should be displayed in the menu for this menuitem.</td>
	</tr>
	<tr>
		<td>url</td>
		<td>-</td>
		<td>no</td>
		<td>
			The URL to open when the user selects the menu item. The url is by default opened in 
			the main window. If 'data-target' is specified, it is instead opened in a named window
			and if 'data-popup' is specified it is opened in a popup window.
		</td>
	</tr>
	<tr>
		<td>data-target</td>
		<td>-</td>
		<td>no</td>
		<td>
			Set this to a window name and the given url will be opened in that window instead.
		</td>
	</tr>
	<tr>
		<td>data-popup</td>
		<td>-</td>
		<td>no</td>
		<td>
			To open the given url in a popup window, set this attribute to a comma-separated list
			with three values: a window name, the width and the height of the popup window. Example:
			<code>data-popup="Logout, 300, 200"</code>.
		</td>
	</tr>
	<tr>
		<td>tooltip</td>
		<td>-</td>
		<td>no</td>
		<td>An optional tooltip that is shown as a popup text while the user
		is pointing the mouse over the menuitem. Not supported by all browsers.</td>
	</tr>
	<tr>
		<td>enabled</td>
		<td>true</td>
		<td>no</td>
		<td>
			Use <code>false</code> to display a disabled (greyed-out) menu item. 
		</td>
	</tr>
	<tr>
		<td>visible</td>
		<td>true</td>
		<td>no</td>
		<td>
			Use <code>false</code> to hide this menu item entirely. 
		</td>
	</tr>
	<tr>
		<td>icon</td>
		<td>-</td>
		<td>no</td>
		<td>
			The filename of an icon to display before the title. The icon must 
			be placed in the <code>/images</code> directory. Do not include any path
			information in the icon attribute.
		</td>
	</tr>
	</table>

	@author Nicklas
	@version 2.0
*/
public class Menuitem
	extends StylableTag
{
	/**
		
	*/
	private static final long serialVersionUID = 1431778959423572670L;

	/**
		The parent Menu.
	*/
	private Menu menu = null;

	/**
		The title of the menu item.
	*/
	private String title = null;
	
	/**
		 A tooltip explaining the menu item.
	*/
	private String tooltip = null;
	
	/**
		If the menu item is enabled.
	*/
	private boolean enabled = true;

	/**
		If the menu item is visible.
	*/
	private boolean visible = true;

	/**
		An icon to display.
	*/
	private String icon = null;
	
	public Menuitem()
	{
		super("menuitem interactable");
	}
	
	/*
		Taglib initialization methods
	*/
	public void setTitle(String title)
	{
		this.title = title;
	}
	public String getTitle()
	{
		return title;
	}
	
	public void setUrl(String url)
	{
		setDynamicAttribute(null, "data-url", url);
	}
	
	public void setTooltip(String tooltip)
	{
		this.tooltip = tooltip;
	}
	public String getTooltip()
	{
		return tooltip;
	}
	
	public void setEnabled(boolean enabled)
	{
		this.enabled = enabled;
	}
	public boolean isEnabled()
	{
		return enabled;
	}

	public void setVisible(boolean visible)
	{
		this.visible = visible;
	}
	public boolean isVisible()
	{
		return visible;
	}

	public void setIcon(String icon)
	{
		this.icon = icon;
	}
	public String getIcon()
	{
		return icon;
	}
	
	public void setDynamic(Iterator<DynamicActionAttribute> dynamicAttributes)
	{
		setMoreDynamicAttributes(dynamicAttributes);
	}
	

	@Override
	public int doStartTag() 
		throws JspException
	{
		if (!(getParent() instanceof Menu)) throw new JspTagException("Tag <m:menuitem> must be inside a <m:menu> tag");
		menu = (Menu)getParent();

		if (!isVisible()) return SKIP_BODY;
		boolean isAutoInitialized = getDynamicAttribute("data-url") != null && isEnabled();
		
		StringBuilder sb = new StringBuilder();
		sb.append("<div");
		addIdAndStyles(sb, isEnabled() ? "" : " disabled", isAutoInitialized ? "auto-init" : null);
		if (isAutoInitialized) sb.append(" data-auto-init=\"menuitem\"");
		if (getTooltip() != null) sb.append(" title=\"").append(getTooltip()).append("\"");
		addDynamicAttributes(sb);
		sb.append(">");
		
		String theIcon = getIcon();
		if (theIcon != null)
		{
			if (!theIcon.startsWith("/"))
			{
				theIcon = ((HttpServletRequest)pageContext.getRequest()).getContextPath()+"/images/"+theIcon;
			}
			String iconClass = isEnabled() ? "menuicon" : "menuicon disabled";
			sb.append("<img src=\"").append(theIcon).append("\" alt=\"\" class=\"").append(iconClass).append("\">");
		}
		else
		{
			sb.append("<span class=\"padding\"></span>");
		}
		if (getTitle() != null) sb.append(getTitle());
		sb.append("</div>");
		try
		{
			menu.addMenuitem(sb.toString());
    	}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return SKIP_BODY;
	}
}


