/*
	$Id: Toolbar.java 7703 2019-04-11 07:31:26Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.taglib.table;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;

import net.sf.basedb.clients.web.taglib.Page;
import net.sf.basedb.clients.web.taglib.StylableTag;
import net.sf.basedb.util.Values;
import net.sf.basedb.core.SessionControl;

/**
	This tag creates a toolbar.
	tag.

	<p>
	<b>Syntax:</b>
	<pre>
   &lt;tbl:toolbar
      id=...
      clazz=...
      subclass=...
      style=...
      visible=true|false
   &gt;
</pre>

	<table class="as-grid">
	<caption>Tag attributes</caption>
	<tr>
		<td><b>Attribute</b></td>
		<td><b>Default value</b></td>
		<td><b>Required</b></td>
		<td><b>Description</b></td>
	</tr>
	<tr>
		<td>id</td>
		<td>-</td>
		<td>no</td>
		<td>
			The ID of the toolbar. The value of this attribute
			goes directly into the standard HTML <code>id</code> attribute.
			This means that if you need a reference to the table object in your own
			JavaScript code, you may for example use the following code:
			<pre class="code">
toolbar = document.getElementById('&lt;id&gt;');
</pre>
		</td>
	</tr>
	<tr>
		<td>clazz</td>
		<td>toolbar</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>class</code> attribute. This allows you to apply different styles
			to different tables.
		</td>
	</tr>
	<tr>
		<td>style</td>
		<td>-</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>style</code> attribute. This allows you to apply different styles
			to different tables.
		</td>
	</tr>
	<tr>
		<td>visible</td>
		<td>true</td>
		<td>no</td>
		<td>
			If the button should be visible or not.
		</td>
	</tr>
	</table>
	<p>

	The <code>&lt;tbl:toolbar&gt;</code> may contain the following subtags:
	<ul>
	<li>{@link Button &lt;tbl:button&gt;}: Creates a button on the toolbar
	<li>{@link Label &lt;tbl:label&gt;}: Creates a label on the toolbar
	</ul>

	@author Nicklas
	@version 2.0
*/
public class Toolbar
	extends StylableTag
{

	/**
		
	*/
	private static final long serialVersionUID = -8719987407920036604L;


	/**
		If the toolbar should be visible or not.
	*/
	private boolean visible = true;

	/**
	 	If buttons should display images.
	*/
	private boolean hasImages = true;
	
	/**
	 	If buttons should display text.
	*/
	private boolean hasText = true;

	public Toolbar()
	{
		super("toolbar");
	}
	
	/*
		Taglib initialization methods
	*/
	public void setVisible(boolean visible)
	{
		this.visible = visible;
	}
	public boolean isVisible()
	{
		return visible;
	}

	public boolean hasImages()
	{
		return hasImages;
	}
	
	public boolean hasText()
	{
		return hasText;
	}
	
	/*
		Package methods
	*/
	/*
		Display methods
	*/
	@Override
	public int doStartTag()
		throws JspException
	{
		if (!isVisible()) return SKIP_BODY;
		Page page = (Page)findAncestorWithClass(this, Page.class);
		
		SessionControl sc = page != null ? page.getSessionControl() : null;
		StringBuilder sb = new StringBuilder();

		sb.append("<div");
		addIdAndStyles(sb);
		addDynamicAttributes(sb);
		sb.append("><table><tr>");

		
		if (sc != null)
		{
			hasImages = Values.getBoolean(sc.getUserClientSetting("toolbar.images"), true);
			hasText = Values.getBoolean(sc.getUserClientSetting("toolbar.text"), true);
		}
		
		try
		{
			pageContext.getOut().print(sb.toString());
		}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return EVAL_BODY_INCLUDE;
	}

	@Override
	public int doEndTag()
		throws JspException
	{
		if (!isVisible()) return EVAL_PAGE;
		StringBuilder sb = new StringBuilder();
		sb.append("</tr></table></div>");
		try
		{
			pageContext.getOut().print(sb.toString());
		}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return EVAL_PAGE;
	}


}

