/**
	$Id: BioPlateType.java 7381 2017-05-22 12:35:27Z nicklas $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.util.Set;

import net.sf.basedb.core.data.BioPlateTypeData;
import net.sf.basedb.core.hibernate.TypeWrapper;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;

/**
	This class represents the type of a bio-plate. This information
	can be used together with event definitions to create a 
	work path in the lab. Eg. start with a plate of some type T1,
	then apply event E1 to it to get a new plate with type T2,
	repeat with event E2 to get a plate with type T3, and so on
	until the biomaterial has been hybridized.

	@author Nicklas
	@since 2.16
	@base.modified $Date: 2017-05-22 14:35:27 +0200 (Mon, 22 May 2017) $
*/
public class BioPlateType
	extends BasicItem
	implements Nameable, Removable
{

	/**
		The type of item represented by this class.
		@see Item#BIOPLATETYPE
		@see #getType()
	*/
	public static final Item TYPE = Item.BIOPLATETYPE;

	/**
	 	Creates a new bioplate type.
	 	 
	  	@param dc DbControl used for database access
	  	@return A new BioPlate object
	  	@throws BaseException If anything goes wrong when creating
	*/
	public static BioPlateType getNew(DbControl dc)
		throws BaseException, InvalidDataException
	{
		BioPlateType bp = dc.newItem(BioPlateType.class);
		bp.setName("New bioplate type");		
		return bp;
	}

	/**
	 	Gets a BioPlate type when knowing the item's id
	  	@param dc DbControl used for database access
	  	@param id The item's id.
	  	@return <code>BioPlateType</code> object
	  	@throws ItemNotFoundException If no item with the given id was found
	  	@throws PermissionDeniedException If logged in user doesn't have read 
	  		permission to the requested item
	  	@throws BaseException If there is another error.
	*/
	public static BioPlateType getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		BioPlateType p = dc.loadItem(BioPlateType.class, id);
		if (p == null) throw new ItemNotFoundException("BioPlateType[id="+id+"]");
		return p;
	}
	
	/**
	 	Gets query that selects all available bioplates
	  	@return <code>ItemQuery</code> object
	 */
	public static ItemQuery<BioPlateType> getQuery()
	{
		return new ItemQuery<BioPlateType>(BioPlateType.class);
	}
	
	BioPlateType(BioPlateTypeData data)
	{
		super(data);
	}
	
	@Override
	BioPlateTypeData getData()
	{
		return (BioPlateTypeData)getBasicData();
	}
	/*
		From the Identifiable interface
		-------------------------------------------
	*/
	@Override
	public Item getType()
	{
		return TYPE;
	}	
	//-------------------------------------------
	/*
		From the Nameable interface
		-------------------------------------------
	*/
	@Override
	public String getName()
	{
		return getData().getName();
	}
	@Override
	public void setName(String name)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		NameableUtil.setName(getData(), name);
	}
	@Override
	public String getDescription()
	{
		return getData().getDescription();
	}
	@Override
	public void setDescription(String description)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		NameableUtil.setDescription(getData(), description);
	}
	// -------------------------------------------
	/*
		From the Removable interface
		-------------------------------------------
	*/
	@Override
	public boolean isRemoved()
	{
		return getData().getRemovedBy() != null;
	}
	@Override
	public void setRemoved(boolean removed)
		throws PermissionDeniedException
	{
		checkPermission(removed ? Permission.DELETE : Permission.WRITE);
		getData().setRemovedBy(removed ? getSessionControl().getLoggedInUserId() : null);
	}
	@Override
	public User getRemovedBy()
		throws PermissionDeniedException, ItemNotFoundException 
	{
		Integer userId = getData().getRemovedBy();
		return userId == null ? null : User.getById(getDbControl(), userId);
	}
	// -------------------------------------------
	/*
	 	From the BasicItem class
	 	--------------------------------------
	 */
	/**
	 	Check if there are any <code>BioPlates</code> or 
	 	<code>BioPlateEventDefinitions</code> using this type.
	*/
	@Override
	public boolean isUsed()
	{
		org.hibernate.Session session = getDbControl().getHibernateSession();
		org.hibernate.query.Query<Long> query = HibernateUtil.getPredefinedQuery(session, 
				"GET_BIOPLATES_FOR_BIOPLATETYPE", Long.class, "count(*)");
		/*
		 	SELECT {1}
		 	FROM BioPlateData bpl
		 	WHERE bpl.bioPlateType = :bioPlateType
		 */
		query.setParameter("bioPlateType", getId(), TypeWrapper.H_INTEGER);
		boolean used = HibernateUtil.loadData(query) > 0;
		return used || super.isUsed();
	}
	
	/**
	 	Gets all <code>BioPlates</code> and <code>BioPlateEventDefinitions</code> 
	 	using this type
	*/
	@Override
	public Set<ItemProxy> getUsingItems()
	{
		Set<ItemProxy> using = super.getUsingItems();
		org.hibernate.Session session = getDbControl().getHibernateSession();
		// Bioplates
		org.hibernate.query.Query<Integer> query = HibernateUtil.getPredefinedQuery(session, 
				"GET_BIOPLATES_FOR_BIOPLATETYPE", Integer.class, "bpl.id");
		/*
		 	SELECT {1}
		 	FROM BioPlateData bpl
		 	WHERE bpl.bioPlateType = :bioPlateType
		 */
		query.setParameter("bioPlateType", getId(), TypeWrapper.H_INTEGER);
		addUsingItems(using, Item.BIOPLATE, query);
		return using;
	}
	//-------------------------------------------

	
	/**
		Get the biomaterial type that can be stored on plates of
		this type.
		@return One of {@link Item#SAMPLE}, {@link Item#EXTRACT}
			or null if the plate can store mixed types
	 */
	public Item getBioMaterialType()
	{
		Integer code = getData().getBioMaterialType();
		return code == null ? null : Item.fromValue(code);
	}
	
	/**
		Set the type of biomaterial that can be stored on plates with
		this type. Valid values are {@link Item#SAMPLE}, {@link Item#EXTRACT}, 
		and null. The value can't be changed 
		after the plate type has been saved to the database.
		@param type A biomaterial item type or null
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission or if the plate type has already been saved
			to the database
		@throws InvalidDataException If the type is not a biomaterial type
	*/
	public void setBioMaterialType(Item type)
	{
		checkPermission(Permission.WRITE);
		if (isInDatabase())
		{
			throw new PermissionDeniedException(Permission.WRITE, 
				"bioMaterialType of BioPlateType[id=" + getId() + "]");
		}
		boolean ok = type == null || type == Item.SAMPLE || type == Item.EXTRACT;
		if (!ok)
		{
			throw new InvalidDataException("type must be one of SAMPLE, EXTRACT or null");
		}
		getData().setBioMaterialType(type == null ? null : type.getValue());		
	}
	
	/**
		Get the subtype that biomaterial items should have when placed
		on a plate with this bioplate type.
		@return A subtype or null if not important
		@since 3.0
	*/
	public ItemSubtype getItemSubtype()
	{
		return getDbControl().getItem(ItemSubtype.class, getData().getItemSubtype());
	}
	
	/**
		Set the subtype that biomaterial items should have when placed on a
		plate with the bioplate type. A subtype can only be specified if 
		a biomaterial type has been set and the given subtype must be compatible
		with the biomaterial type.
		@param subtype A subtype or null if the subtype is not important
		@since 3.0
	*/
	public void setItemSubtype(ItemSubtype subtype)
	{
		checkPermission(Permission.WRITE);
		
		if (subtype == null)
		{
			getData().setItemSubtype(null);
		}
		else
		{
			Item bmType = getBioMaterialType();
			if (bmType == null)
			{
				throw new InvalidDataException("Can't set subtype when no biomaterial type has been set: " + this.getName());
			}
			if (bmType != subtype.getMainItemType())
			{
				throw new InvalidDataException("The subtype is not used for " + bmType.name() + ": " + subtype.getName());
			}
			subtype.checkPermission(Permission.USE);
			getData().setItemSubtype(subtype.getData());
		}
	}
	
	
	/**
		Get the subtype of storage location where plates of this type are usually stored.
		The storage type should be a subtype for hardware items.
		@return A subtype or null if not important
		@since 3.2
	*/
	public ItemSubtype getStorageType()
	{
		return getDbControl().getItem(ItemSubtype.class, getData().getStorageType());
	}
	
	/**
		Set the subtype of storage location where plates of this type are usually stored.
		The storage type should be a subtype for hardware items.
		@param storageType A subtype or null if the subtype is not important
		@since 3.2
	*/
	public void setStorageType(ItemSubtype storageType)
	{
		checkPermission(Permission.WRITE);
		
		if (storageType == null)
		{
			getData().setStorageType(null);
		}
		else
		{
			if (storageType.getMainItemType() != Item.HARDWARE)
			{
				throw new InvalidDataException("The storage type is not used for HARDWARE: " + storageType.getName());
			}
			storageType.checkPermission(Permission.USE);
			getData().setStorageType(storageType.getData());
		}
	}

	/**
		Get the lock mode for wells located on plates of this plate type.
		The lock mode determines if biomaterial may be changed in the
		well.
		
		@return A lock mode
	*/
	public BioWell.LockMode getLockMode()
	{
		return BioWell.LockMode.fromValue(getData().getLockMode());
	}
	/**
		Set the lock mode for wells on plates of this plate type.
		@param lockMode A lock mode, null is not allowed
	*/
	public void setLockMode(BioWell.LockMode lockMode)
	{
		checkPermission(Permission.WRITE);
		if (lockMode == null) throw new InvalidUseOfNullException("lockMode");
		getData().setLockMode(lockMode.getValue());
	}
	
	/**
		Creates a query that returns all bioplates that are using
		this type.
		@return An ItemQuery object
	*/
	public ItemQuery<BioPlate> getBioPlates()
	{
		ItemQuery<BioPlate> query = BioPlate.getQuery();
		query.restrictPermanent(
				Restrictions.eq(
					Hql.property("bioPlateType"), 
					Hql.entity(this)
				)
			);
		return query;
	}

	
}
