/*
	$Id: ExtendedPropertyAccessor.java 7381 2017-05-22 12:35:27Z nicklas $

	Copyright (C) 2005 Samuel Andersson, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.ExtendableData;

import org.hibernate.property.access.spi.PropertyAccess;
import org.hibernate.property.access.spi.PropertyAccessStrategy;
import org.hibernate.property.access.spi.Getter;
import org.hibernate.property.access.spi.Setter;
import org.hibernate.PropertyNotFoundException;
import org.hibernate.engine.spi.SharedSessionContractImplementor;
import org.hibernate.engine.spi.SessionFactoryImplementor;

import java.lang.reflect.Member;
import java.lang.reflect.Method;
import java.util.Map;

/**
	This class is used to interface with Hibernate for extended
	properties. We need specialized getter and setter methods since
	there are no regulare get and set methods for the extended
	properties. Ie. we have to call <code>getExtended("name")</code>
	instead of <code>getName()</code>.
	<p>
	During startup Hibernate creates a pair of <code>Getter</code>
	and <code>Setter</code> objects for each extended property of an
	<code>Extendable</code> item. These objects are then used to access the
	extended properties for all items via the {@link Getter#get(Object)
	GetterSetter.get} and {@link Setter#set(Object,Object,SessionFactoryImplementor)
	GetterSetter.set} methods.

	@author Nicklas, Samuel
	@version 2.0
	@see net.sf.basedb.core.data.ExtendableData
	@see ExtendedProperty
	@see ExtendedProperties
*/
public final class ExtendedPropertyAccessor
	implements PropertyAccessStrategy
{

	/**
	  	Create a new ExtendedPropertyAccessor object.
	*/
	public ExtendedPropertyAccessor()
	{}

	@SuppressWarnings("rawtypes")
	@Override
	public PropertyAccess buildPropertyAccess(Class clazz, String property) 
	{
		if (ExtendableData.class.isAssignableFrom(clazz))
		{
			return new GetterSetter(this, clazz, property);
		}
		else
		{
			throw new PropertyNotFoundException(clazz.getName()+" is not Extendable");
		}
	}

	/**
		This class implements the Hibernate <code>Getter</code> and
		<code>Setter</code> interfaces. For each property of
		an <code>Extendable</code> item there exist one <code>Getter</code>
		and <code>Setter</code> object.
	*/
	static final class GetterSetter
		implements PropertyAccess, Getter, Setter
	{

		private static final long serialVersionUID = 2904387621348099703L;
		private final PropertyAccessStrategy strategy;
		private final Class<?> clazz;
		private final String property;

		/**
			Create a new getter or setter.
			@param clazz The <code>Class</code> of the item
			@param property The name of the extended property
		*/
		GetterSetter(PropertyAccessStrategy strategy, Class<?> clazz, String property)
		{
			this.strategy = strategy;
			this.clazz = clazz;
			this.property = property;
		}

		/*
			From the PropertyAccess interface
			-------------------------------------------
		*/
		@Override
		public PropertyAccessStrategy getPropertyAccessStrategy() 
		{
			return strategy;
		}

		@Override
		public Getter getGetter() 
		{
			return this;
		}

		@Override
		public Setter getSetter() 
		{
			return this;
		}

		
		/*
			From the Getter interface
			-------------------------------------------
		*/
		/**
			Get the value of the extended property.
			@param target The <code>ExtendableData</code> that holds the value
			@throws ClassCastException If the target is not
				<code>ExtendableData</code>
		*/
		@Override
		public Object get(Object target)
		{
			ExtendableData e = (ExtendableData)target;
			return e.getExtended(property);
		}
		/**
			Get the value of the extended property.
			@param target The <code>ExtendableData</code> that holds the value
			@param mergeMap A map of merged persistent instances to detached instances, whatever
				that means...
			@param session The session from which this request originated.
			@throws ClassCastException If the target is not
				<code>ExtendableData</code>
		*/
		@SuppressWarnings("rawtypes")
		@Override
		public Object getForInsert(Object target, Map mergeMap, SharedSessionContractImplementor session)
		{
			return get(target);
		}
		// -------------------------------------------
		/*
			From the Setter interface
			-------------------------------------------
		*/
		/**
			Set the value of the extended property.
			@param target The <code>Extendable</code> that holds the value
			@param value The new value
			@param factory The session factory from which this request originated.
			@throws ClassCastException If the target is not
				<code>ExtendableData</code>
		*/
		@Override
		public void set(Object target, Object value, SessionFactoryImplementor factory)
		{
			ExtendableData e = (ExtendableData)target;
			e.setExtended(property, value);
		}
		// -------------------------------------------
		/*
			From the Getter and Setter interface
			-------------------------------------------
		*/
		/**
			Always return null.
		*/
		@Override
		public Member getMember()
		{
			return null;
		}
		/**
			Always return null.
		*/
		@SuppressWarnings("rawtypes")
		@Override
		public Class getReturnType()
		{
			return null;
		}
		/**
			Always return null.
		*/
		@Override
		public String getMethodName()
		{
			return null;
		}
		/**
			Always return null.
		*/
		@Override
		public Method getMethod()
		{
			return null;
		}
		// -------------------------------------------

	}

}

