/*
	$Id $

	Copyright (C) 2016 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.query.Query;
import net.sf.basedb.core.query.QueryElement;
import net.sf.basedb.core.query.QueryType;
import net.sf.basedb.core.query.Expression;
import net.sf.basedb.util.EqualsHelper;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;

/**
	An expression representing the property of a raw bioassay.
	<p>
	Throws an <code>UnsupportedOperationException</code> if 
	{@link Query#getQueryType()} isn't {@link QueryType#SQL}.

	@author Nicklas
	@since 3.9
	@see net.sf.basedb.core.query.Dynamic#rawBioAssay(String)
*/
public class RawBioAssayPropertyExpression
	implements Expression
{
	private static volatile Map<String, String> propertyToColumn;

	private final String property;
	
	private final String column;
	
	/**
		Create a new raw bioassay property expression.
		@param property The name of the property
	*/
	public RawBioAssayPropertyExpression(String property)
		throws InvalidDataException
	{
		this.property = property;
		this.column = getColumn(property);
		if (column == null) 
		{
			throw new InvalidDataException("Unknown property '"+property+"' of raw bioassay.");
		}
	}

	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	@Override
	public String toQl(Query query, DbControl dc)
		throws BaseException
	{
		if (query.getQueryType() == QueryType.SQL)
		{
			String joinAlias = RealTable.RAWBIOASSAYS.getAlias();
			return HibernateUtil.quote(joinAlias) + "." + HibernateUtil.quote(column); 
		}
		else
		{
			throw new UnsupportedOperationException("'rawBioAssayProperty' operation not supported by "+query.getQueryType()+" query.");
		}
	}
	/**
		@return Always null
	*/
	@Override
	public Collection<? extends QueryElement> getChildren()
	{
		return null;
	}
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	@Override
	public String toString()
	{
		return RealTable.RAWBIOASSAYS.getAlias() + "." + property; 
	}
	/**
		This expression is equal to another RawBioAssayPropertyExpression if they 
		represent the same reporter property.
	*/
	@Override
	public boolean equals(Object other)
	{
		if (this == other) return true;
		if (other == null || this.getClass() != other.getClass()) return false;
		RawBioAssayPropertyExpression o = (RawBioAssayPropertyExpression)other;
		return EqualsHelper.equals(property, o.property);
	}
	@Override
	public int hashCode()
	{
		return EqualsHelper.hashCode(property);
	}
	// -------------------------------------------
	
	@SuppressWarnings("unchecked")
	private static String getColumn(String property)
	{
		if (propertyToColumn == null)
		{
			synchronized (RawBioAssayPropertyExpression.class)
			{
				// So we are really sure no other thread created this while we were waiting
				if (propertyToColumn == null)
				{
					propertyToColumn = new HashMap<String, String>();
					org.hibernate.mapping.PersistentClass pc = HibernateUtil.getClassMapping("net.sf.basedb.core.data.RawBioAssayData");
					Iterator<org.hibernate.mapping.Property> propIt = (Iterator<org.hibernate.mapping.Property>)pc.getPropertyIterator();
					while (propIt.hasNext())
					{
						org.hibernate.mapping.Property prop = propIt.next();
						Iterator<org.hibernate.mapping.Column> colIt = (Iterator<org.hibernate.mapping.Column>)prop.getColumnIterator();
						if (colIt.hasNext())
						{
							org.hibernate.mapping.Column col = colIt.next();
							propertyToColumn.put(prop.getName(), col.getName());
						}
					}
					propertyToColumn.put("id", "id"); //Add id as a property because hibernate wont
				}
			}
		}
		return propertyToColumn.get(property);
	}
}
