/**
	$Id: MultiFormatter.java 7649 2019-03-14 06:52:48Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.formatter;

import java.util.HashMap;
import java.util.Map;

import net.sf.basedb.util.excel.ExcelFormatter;
import net.sf.basedb.util.excel.ExcelValue;


/**
 	A generic formatter that can be used to format objects of mixed classes.
 	To use it you register a formatter for each class you expect to use. 
 	A default formatter is used for objects with unregistered classes.
 	<p>
 	Null values return the empty string.
 	
	@author nicklas
	@version 2.0
	@base.modified $Date: 2019-03-14 07:52:48 +0100 (Thu, 14 Mar 2019) $
*/
public class MultiFormatter
	implements Formatter<Object>, ExcelFormatter<Object, Object>
{

	private final Formatter<Object> defaultFormatter;
	private final boolean checkSuperclass;
	private final Map<Class<?>, Formatter<Object>> formatters;
	private final boolean decideOnFirst;
	private Formatter<Object> formatter;
	
	/**
		Create a new multi formatter.
		@param defaultFormatter The formatter to use if the object's class
			isn't found among the registered formatters
		@param checkSuperclass If the superclass and interfaces of the object should be
			checked in case there is no exact match
	*/
	public MultiFormatter(Formatter<Object> defaultFormatter, boolean checkSuperclass)
	{
		this(defaultFormatter, checkSuperclass, false);
	}
	
	/**
		Create a new multi formatter.
		@param defaultFormatter The formatter to use if the object's class
			isn't found among the registered formatters
		@param checkSuperclass If the superclass and interfaces of the object should be
			checked in case there is no exact match
		@param decideOnFirst If TRUE, the first non-null value decides which parent
			formatter to use. All subsequent calls to {@link #format(Object)} uses
			the same formatetter. This is useful when you know that the formatter will
			only use the same class of objects, but not which class.
		@since 2.7.2
	*/
	public MultiFormatter(Formatter<Object> defaultFormatter, boolean checkSuperclass, 
		boolean decideOnFirst)
	{
		this.defaultFormatter = defaultFormatter;
		this.formatters = new HashMap<Class<?>, Formatter<Object>>();
		this.checkSuperclass = checkSuperclass;
		this.decideOnFirst = decideOnFirst;
	}
	
	/*
		From the Formatter interface
		-------------------------------------------
	*/
	@Override
	public String format(Object value)
	{
		if (value == null) return "";
		Formatter<Object> f = getFormatter(value);
		return f.format(value);
	}
	@Override
	public Number parseString(String value)
	{
		throw new UnsupportedOperationException("parseString");
	}
	// -------------------------------------------

	private Formatter<Object> getFormatter(Object value)
	{
		Class<?> c = value.getClass();
		Formatter<Object> f = formatter;
		if (f == null)
		{
			f = formatters.get(c);
			if (f == null && checkSuperclass)
			{
				c = c.getSuperclass();
				Class<?>[] interfaces = c.getInterfaces();
				while (f == null && c != null)
				{
					f = formatters.get(c);
					c = c.getSuperclass();
				}
				if (f == null)
				{
					for (Class<?> iface : interfaces)
					{
						f = formatters.get(iface);
						if (f != null) break;
					}
				}
			}
			if (f == null) f = defaultFormatter;
			if (decideOnFirst) formatter = f;
		}
		return f;
	}
	
	/**
		Register a formatter for the specified class.
		@param clazz The class of objects to format with the formatter
		@param formatter The formatter
	*/
	@SuppressWarnings({ "unchecked", "rawtypes" })
	public <T> void registerFormatter(Class<? extends T> clazz, Formatter<T> formatter)
	{
		formatters.put((Class)clazz, (Formatter)formatter);
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	@Override
	public ExcelValue<Object> toExcelValue(Object value) 
	{
		if (value == null) return null;
		Formatter<Object> f = getFormatter(value);
		ExcelValue ev = null;
		if (f instanceof ExcelFormatter)
		{
			ExcelFormatter ef = (ExcelFormatter)f;
			ev = ef.toExcelValue(value);
		}
		else
		{
			ev = ExcelValue.asString(f.format(value));
		}
		return ev;
	}
}
