/**
	$Id$

	Copyright (C) 2015 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.listable;

import java.util.HashSet;
import java.util.Set;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Extract;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;

/**
	Source item transformer implementation that transform extract items to their
	child extract items. From the given set of extract ID:s the transformer will
	recursively load as many child extracts as possible. The source items
	may optionally be included in the target set.

	@author Nicklas
	@since 3.5
*/
public class ExtractToChildExtractTransformer 
	extends AbstractSourceItemTransformer
{

	private final boolean includeSourcesInTarget;
	private final boolean pushOnly;
	
	/**
		Create a new extract to child extract transformer.
		@param includeSourcesInTarget TRUE to include the source items in the target
	*/
	public ExtractToChildExtractTransformer(boolean includeSourcesInTarget) 
	{
		this(includeSourcesInTarget, false);
	}
	/**
		Create a new extract to child extract transformer that only load children
		has a subtype with "push annotations" set. 
		@since 3.19.5
	*/
	public ExtractToChildExtractTransformer(boolean includeSourcesInTarget, boolean childrensThatPushOnly)
	{
		super(Item.EXTRACT, Item.EXTRACT);
		this.includeSourcesInTarget = includeSourcesInTarget;
		this.pushOnly = childrensThatPushOnly;
	}

	@Override
	public Set<Integer> transform(TransformContext context, Set<Integer> source) 
	{
		DbControl dc = context.getDbControl();
		
		ItemQuery<Extract> query = Extract.getQuery();
		query.setIncludes(context.getInclude());
		// Extracts with an extract as parent
		query.restrict(Restrictions.eq(Hql.property("parentType"), Expressions.integer(Item.EXTRACT.getValue())));
		query.join(Hql.innerJoin("creationEvent", "ce"));
		query.join(Hql.innerJoin("ce", "sources", "sbm"));
		query.join(Hql.innerJoin("sbm", "bioMaterial", "bm"));
		query.restrict(
			Restrictions.in(
				Hql.alias("bm"), 
				Expressions.parameter("parents")
			));
		if (pushOnly)
		{
			query.join(Hql.innerJoin("itemSubtype", "st"));
			query.restrict(Restrictions.eq(Hql.property("st", "pushAnnotations"), Expressions.bool(true)));
		}

		// Keep track of all seen children and parents
		Set<Integer> allParents = new HashSet<Integer>();
		Set<Integer> allChildren = new HashSet<Integer>();
		if (includeSourcesInTarget) allChildren.addAll(source);

		// Parents that we have not yet checked
		Set<Integer> parents = source;
		while (parents.size() > 0)
		{
			context.collect(getSourceItemType(), parents);
			Set<Integer> children = context.avoid(getTargetItemType(), safeIdList(dc, query, "parents", parents));
			
			// Store new children and "used" parents
			allChildren.addAll(children);
			allParents.addAll(parents);
			
			// Remove parents that we have already seen...
			// ...the remaining children become our new parents
			children.removeAll(allParents);
			parents = children;
		}
		
		return allChildren;
	}


}
