/**
	$Id$

	Copyright (C) 2020 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.listable;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import net.sf.basedb.core.Item;

/**
	Transformer implementation that can be used to to find parent items
	to children that have a subtype with the "push annotations to parent"
	flag set.
	
	Example: The source list is extracts of a subtype with this flag.
	This transformer will find all parent items to source list. If
	any of the parent items also has a subtype with the "push annotations" flag
	to process is repeated.
	
	This implemntatation works with SAMPLE or EXTRACT types only. If starting with a
	SAMPLE list it will also move up to BIOSOURCE and then back down one step.
	If starting with a EXTRACT list it will move up to SAMPLE and BIOSOURCE (if
	all intermediate steps have the "push annotations" flag).
	
	If used in a chain, this transformation should be used first in the chain.
	
	@since 3.16
*/
public class PushToParentsTransformer 
	implements SourceItemTransformer
{

	private final Item itemType;
	
	public PushToParentsTransformer(Item itemType)
	{
		this.itemType = itemType;
	}
	
	@Override
	public Item getSourceItemType() 
	{
		return itemType;
	}

	@Override
	public Item getTargetItemType() 
	{
		return itemType;
	}

	@Override
	public Set<Integer> transform(TransformContext context, Set<Integer> source) 
	{
		if (source.size() == 0) return Collections.emptySet();
		
		Set<Integer> result = new HashSet<>();
		if (itemType == Item.EXTRACT)
		{
			// Get parent extracts to sources with "push annotations"
			Set<Integer> allExtracts = new ExtractToParentExtractTransformer(true, true).transform(context, source);
			result.addAll(allExtracts);
			
			// Get parent samples to extracts with "push annotations"
			Set<Integer> samples = new ExtractToSampleTransformer(true).transform(context, allExtracts);
			if (samples.size() > 0)
			{
				samples = getAllSamples(context, samples);
				// Move back down to extracts again since we need to finish with a set of extracts
				samples = new SampleToChildSampleTransformer(true).transform(context, samples);
				result.addAll(new SampleToExtractTransformer().transform(context, samples));
			}				
		}
		else if (itemType == Item.SAMPLE)
		{
			// Get parent samples to sources with "push annotations"
			result.addAll(getAllSamples(context, source));
		}
		
		return result;
	}

	
	private Set<Integer> getAllSamples(TransformContext context, Set<Integer> source)
	{
		Set<Integer> allSamples = new SampleToParentSampleTransformer(true, true).transform(context, source);
		Set<Integer> bioSources = new SampleToBioSourceTransformer(true).transform(context, allSamples);
		if (bioSources.size() > 0)
		{
			allSamples.addAll(new BioSourceToSampleTransformer().transform(context, bioSources));
		}
		return allSamples;
	}
	
}
