/**
	$Id$

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.overview.loader;

import java.util.Collections;
import java.util.List;

import net.sf.basedb.core.Annotatable;
import net.sf.basedb.core.Annotation;
import net.sf.basedb.core.AnyToAny;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.FileSetMember;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.PermissionDeniedException;
import net.sf.basedb.core.Protocol;
import net.sf.basedb.core.snapshot.AnnotationSnapshot;
import net.sf.basedb.core.snapshot.SnapshotManager;
import net.sf.basedb.util.overview.OverviewContext;
import net.sf.basedb.util.overview.node.AnnotationNameGenerator;
import net.sf.basedb.util.overview.node.ChildNodeDirection;
import net.sf.basedb.util.overview.Node;
import net.sf.basedb.util.overview.node.NodeFactory;

/**
	Node loader implementation for annotations. Annotations are
	loaded as an end-point folder node with one child entry
	for each annotation. It can never be root node or a forward/reverse-loading 
	node.

	@author Nicklas
	@version 2.10
	@base.modified $Date$
*/
public class AnnotationLoader
	extends BasicItemNodeLoader<Annotation>
{

	/**
		Get the annotatable item from the node. This is in most cases
		the {@link Node#getItem()} item, but can also be the target
		item of an {@link AnyToAny} node or the file of a {@link FileSetMember}
		node.
		@since 3.17
	*/
	public static Annotatable getAnnotatableFromNode(DbControl dc, Node node)
	{
		Annotatable annotatable = null;
		Item nodeType = node.getItemType();
		try
		{
			if (nodeType == Item.ANYTOANY)
			{
				// The parent item is the 'to' target of the any-to-any
				AnyToAny any = (AnyToAny)node.getItem(dc);
				annotatable = (Annotatable)any.getTo();
			}
			else if (nodeType == Item.FILESETMEMBER)
			{
				// The parent item is the 'file' item
				FileSetMember member = (FileSetMember)node.getItem(dc);
				annotatable = member.getFile();
			}
			else
			{
				annotatable = (Annotatable)node.getItem(dc);
			}
		}
		catch (PermissionDeniedException ex)
		{}
		return annotatable;
	}
	
	public AnnotationLoader()
	{
		super(Item.ANNOTATION, DENY_ROOT_NODE, new AnnotationNameGenerator());
	}

	/*
		From the NodeLoader interface
		------------------------------
	*/
	@Override
	public Node createPropertyNode(DbControl dc, OverviewContext context, Node parentNode)
	{
		NodeFactory<Annotation> nf = getNodeFactory(dc, context);
		Annotatable parent = getAnnotatableFromNode(dc, parentNode);

		Node folderNode = null;
		if (parent != null && parent.isAnnotated())
		{
			// Load the protocol, if any, so that we can ignore annotations
			// that are protocol parameters
			Protocol protocol = null;
			try
			{
				protocol = parent.getProtocol();
			}
			catch (PermissionDeniedException ex)
			{}
			
			folderNode = new Node("annotations", "Annotations", parentNode, ChildNodeDirection.NONE);
			SnapshotManager snapshotManager = context.getSnapshotManager();
			
			List<AnnotationSnapshot> all = snapshotManager.findAnnotations(dc, parent, null, true);
			Collections.sort(all, AnnotationSnapshot.sortByAnnotationType(dc));
			for (AnnotationSnapshot a : all)
			{
				Annotation ann = a.getThisAnnotation(dc);
				// Annotations that are protocol parameters are handled by ProtocolParameterLoader				
				if (a.getSource() == Annotation.Source.PRIMARY && protocol != null && protocol.isParameter(a.getAnnotationType(dc)))
				{
					/* #### CONTINUE-STATEMENT #### */
					continue;				
				}
				createItemNode(nf, ann, null, false, folderNode, ChildNodeDirection.NONE);
			}
			//folderNode.sortChildren(null);
		}
		postValidateFolder(nf, folderNode, parentNode, false);
		return folderNode;
	}
	// --------------------------------
}
