/**
	$Id: PerformanceTimer.java 6127 2012-09-14 07:28:35Z nicklas $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.timer;

/**
	A simple timer for measuring code performance. You can register
	one or more timers when constructing an object of this class. Typically,
	there should be one timer for each part of the code (method call) that
	you want to measure.
	Use {@link #start(int)}, {@link #stop(int)} and {@link #stopStart(int, int)}
	to start and stop the timers in your code. Example:
	
<pre class="code">
PerformanceTimer timer =
  new PerformanceTimer("method1()", "method2()");
timer.start(0);
method1();
timer.stopStart(0, 1);
method2();
timer.stop(1);
System.out.println(timer.toString());
</pre>

	This classes use {@link System#nanoTime()} to get the time. All times
	are returned as nano-second differences between the start and stop time
	of the timer. Starting and stopping a timer multiple times adds more
	time to the timer. Use {@link #reset(int)} to zero a timer.

	@author nicklas
	@version 2.4
	@base.modified $Date: 2012-09-14 09:28:35 +0200 (Fri, 14 Sep 2012) $
*/
public class PerformanceTimer
{

	private final String[] names;
	private final long[] timers;
	private final boolean[] isRunning;
	/**
		Create a new timer object for measuring one or more times.
		
		@param names The name of the timers
	*/
	public PerformanceTimer(String... names)
	{
		this.names = names;
		this.timers = new long[names.length];
		this.isRunning = new boolean[names.length];
	}
	
	/**
		Start the timer with the given index. This method does nothing
		if the timer is already running.
		
		@param index The index of the timer
	*/
	public void start(int index)
	{
		if (isRunning[index]) return;
		timers[index] -= System.nanoTime();
		isRunning[index] = true;
	}
	
	/**
		Stop a running timer. This method does nothing if the timer
		isn't running.
		
		@param index The index of the timer
		@return The current time
	*/
	public long stop(int index)
	{
		if (isRunning[index]) timers[index] += System.nanoTime();
		isRunning[index] = false;
		return timers[index];
	}
	
	/**
		Start one timer and stop another.
		@param stopIndex The timer to stop
		@param startIndex The timer to start
		@return The current time of the stopped timer
	*/
	public long stopStart(int stopIndex, int startIndex)
	{
		start(startIndex);
		return stop(stopIndex);
	}
	
	/**
		Stop all running timers
	*/
	public void stopAll()
	{
		for (int i = 0; i < timers.length; ++i)
		{
			stop(i);
		}
	}
	
	/**
		Check if a timer is running or not.
		@param index The index of the timer
		@return TRUE if the timer is running, FALSE if it has stopped
	*/
	public boolean isRunning(int index)
	{
		return isRunning[index];
	}
	
	/**
		Get the current time of a timer. 
		@param index The index of the timer
		@return The current time of the timer
	*/
	public long getTime(int index)
	{
		return timers[index] + (isRunning[index] ? System.nanoTime() : 0);
	}
	
	/**
		Reset a timer.
		@param index The timer to reset
	*/
	public void reset(int index)
	{
		timers[index] = isRunning[index] ? -System.nanoTime(): 0;
	}
	
	/**
		Reset all timers.
	*/
	public void resetAll()
	{
		for (int i = 0; i < timers.length; ++i)
		{
			reset(i);
		}
	}
	
	/**
		Print out time information about each timer. The times
		are rounded to millisecond values.
	*/
	@Override
	public String toString()
	{
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < timers.length; ++i)
		{
			sb.append(names[i]).append("\t").append(getTime(i) / 1000000).append(" ms");
			if (isRunning[i])
			{
				sb.append(" (running)");
			}
			sb.append("\n");
		}
		return sb.toString();
	}
	
}
