/**
	$Id: CloneReportersPlugin.java 5886 2011-11-23 13:32:22Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.plugins;

import java.util.Arrays;
import java.util.Collection;
import java.util.EnumSet;
import java.util.HashSet;
import java.util.Set;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Experiment;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.ProgressReporter;
import net.sf.basedb.core.ReporterCloneBatcher;
import net.sf.basedb.core.ReporterCloneTemplate;
import net.sf.basedb.core.ReporterCloneBatcher.CloneSource;
import net.sf.basedb.core.plugin.AbstractPlugin;
import net.sf.basedb.core.plugin.Permissions;
import net.sf.basedb.core.plugin.Request;
import net.sf.basedb.core.plugin.Response;
import net.sf.basedb.core.signal.SignalHandler;
import net.sf.basedb.core.signal.SignalTarget;
import net.sf.basedb.core.signal.ThreadSignalHandler;

/**
	A plug-in that clone reporter annotations to an experiment. To use, create
	a {@link Job} and make sure the {@link Job#getExperiment()} is set. 
	Set the "template" parameter to a {@link ReporterCloneTemplate} item to use
	that as template for the cloning operation, or use null to clone all reporter
	annotations. Set the "cloneSource" parameter to a name from the {@link CloneSource}
	enumeration to use a specific reporter source.

	@author Nicklas
	@since 3.1
	@base.modified $Date: 2011-11-23 14:32:22 +0100 (Wed, 23 Nov 2011) $
*/
public class CloneReportersPlugin
	extends AbstractPlugin
	implements SignalTarget
{

	
	private ThreadSignalHandler signalHandler;
	
	public CloneReportersPlugin()
	{}
	
	/*
		From the Plugin interface
		-------------------------
	*/
	@Override
	public MainType getMainType()
	{
		return MainType.OTHER;
	}
	@Override
	public boolean supportsConfigurations()
	{
		return false;
	}
	@Override
	public boolean requiresConfiguration()
	{
		return false;
	}
	/**
	 	Request write permission to Experiment:s and Reporter clone
	 	templates and read permission to reporters.
	*/
	@Override
	public Collection<Permissions> getPermissions()
	{
		Set<Permissions> permissions = new HashSet<Permissions>();
		permissions.add(new Permissions(Item.EXPERIMENT, null, EnumSet.of(Permission.WRITE)));
		permissions.add(new Permissions(Item.REPORTERCLONETEMPLATE, null, EnumSet.of(Permission.WRITE)));
		permissions.add(new Permissions(Item.REPORTER, null, EnumSet.of(Permission.READ)));
		permissions.add(new Permissions(Item.JOB, null, EnumSet.of(Permission.READ)));
		return permissions;
	}

	@Override
	public void run(Request request, Response response, ProgressReporter progress)
	{
		if (signalHandler != null) signalHandler.setWorkerThread(null);
		
		DbControl dc = null;
		try
		{
			dc = sc.newDbControl();
			Job theJob = dc.loadItem(job.getJob());
			Experiment experiment = theJob.getExperiment();
			ReporterCloneTemplate template = dc.loadItem((ReporterCloneTemplate)job.getValue("template"));
			String tmp = (String)job.getValue("cloneSource");
			CloneSource source = tmp == null ? CloneSource.AUTO : CloneSource.valueOf(tmp);
			ReporterCloneBatcher batcher = experiment.getReporterCloneBatcher(dc);
			int numReporters = batcher.cloneReporters(source, template, progress);
			if (theJob.getDescription() != null)
			{
				experiment.getVirtualDb().getReporterCloneTemplate().setDescription(theJob.getDescription());
			}
			dc.commit();
			response.setDone("Cloned " + numReporters + " reporters");
		}
		catch (Throwable t)
		{
			response.setError(t.getMessage(), Arrays.asList(t));
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	// -------------------------
	/*
		From the SignalTarget interface
		-------------------------------------------
	*/
	@Override
	public SignalHandler getSignalHandler()
	{
		signalHandler = new ThreadSignalHandler();
		return signalHandler;
	}
	// -------------------------------------------

}
