/**
	$Id$

	Copyright (C) 2015 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.plugins.batchimport;

import java.util.Collections;
import java.util.List;
import java.util.Set;

import net.sf.basedb.core.BioSource;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.PluginParameter;
import net.sf.basedb.core.Tag;
import net.sf.basedb.core.plugin.GuiContext;
import net.sf.basedb.util.parser.FlatFileParser;
import net.sf.basedb.util.parser.Mapper;

/**
	Plug-in for importing tag items in a batch. The plug-in can create new 
	items and update existing items.

	@author nicklas
	@since 3.15.2
*/
public class TagImporter
	extends AbstractItemImporter<Tag>
{

	private static final Set<GuiContext> guiContexts =  
		Collections.singleton(new GuiContext(Item.TAG, GuiContext.Type.LIST));
	
	public TagImporter()
	{}

	/*
		From the InteractivePlugin interface
		------------------------------------
	*/
	@Override
	public Set<GuiContext> getGuiContexts()
	{
		return guiContexts;
	}
	// ------------------------------------

	/*
		From the AbstractItemImporter class
		------------------------------------
	*/
	
	private Mapper nameMapper;
	private Mapper descriptionMapper;
	private Mapper registeredMapper;
	
	/**
		Useable methods are:
		<ul>
		<li>{@link PropertyIdMethod#NAME}
		<li>{@link InternalIdMethod#INTERNAL_ID}
		<li>{@link FallbackIdMethod#NAME_OR_ID}
		</ul>
	*/
	@Override
	protected IdMethod[] getIdMethods()
	{
		return new IdMethod[] { PropertyIdMethod.NAME,  
				InternalIdMethod.INTERNAL_ID, FallbackIdMethod.NAME_OR_ID};
	}
	
	@Override
	protected Item getItemForSubtypes()
	{
		return Item.TAG;
	}
	
	/**
		Enable annotations as item identifier.
	*/
	@Override
	protected Item getItemForAnnotationTypes()
	{
		return Item.KIT;
	}

	/**
		Adds column mappings for name and description.
	*/
	@Override
	protected void addMoreColumnMappingParameters(List<PluginParameter<?>> parameters)
	{
		parameters.add(internalIdColumnMapping);
		parameters.add(nameColumnMapping);
		parameters.add(subtypeColumnMapping);
		parameters.add(idAnnotationColumnMapping);
		parameters.add(registeredColumnMapping);
		parameters.add(descriptionColumnMapping);
		parameters.add(permissionTemplateColumnMapping);
	}
	/**
		Calls {@link Tag#getQuery()}.
	*/
	@Override
	protected ItemQuery<Tag> createItemQuery()
	{
		return Tag.getQuery();
	}
	@Override
	protected void createColumnMappers(FlatFileParser ffp, boolean cropStrings)
	{
		nameMapper = getMapper(ffp, (String)job.getValue("nameColumnMapping"), 
				cropStrings ? BioSource.MAX_NAME_LENGTH : null, null);
		descriptionMapper = getMapper(ffp, (String)job.getValue("descriptionColumnMapping"), 
				cropStrings ? BioSource.MAX_DESCRIPTION_LENGTH : null, null);
		registeredMapper = getMapper(ffp, (String)job.getValue("registeredColumnMapping"), null, null);
	}
	@Override
	protected Tag createItem(DbControl dc, FlatFileParser.Data data)
	{
		Tag tag = Tag.getNew(dc);
		if (registeredMapper != null) 
		{
			tag.setEntryDate(registeredMapper.getDate(data));
		}
		updateItem(dc, tag, data);
		return tag;
	}
	@Override
	protected void updateItem(DbControl dc, Tag tag, FlatFileParser.Data data)
	{
		if (nameMapper != null) tag.setName(nameMapper.getString(data));
		if (descriptionMapper != null) tag.setDescription(descriptionMapper.getString(data));

		updateItemSubtype(dc, tag, data);
	}
	// ------------------------------------
}
