/*
	$Id: TestPhysicalBioAssay.java 5739 2011-09-15 07:18:14Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.*;

import java.util.Date;
import java.util.Set;

public class TestPhysicalBioAssay
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing physical bioassays");
		write_header();
		// Standard tests: create, load, list
		int id = test_create(0, "Hyb #1",SystemItems.getId(PhysicalBioAssay.HYBRIDIZATION), 1);
		int id2 = test_create(0,"Flow cell #2",  SystemItems.getId(PhysicalBioAssay.CLONAL_AMPLIFICATION), 1);
		
		test_load(id);

		int arrayDesignId = TestArrayDesign.test_create(Platform.GENERIC, false);
		int arrayBatchId = TestArrayBatch.test_create(arrayDesignId, false);
		int arraySlideId = TestArraySlide.test_create(arrayBatchId, false);
		int id3 = test_create(arraySlideId, "Test #3", 0, 1);
		
		int cy3 = TestTag.test_load("cy3");
		int cy5 = TestTag.test_load("cy5");
		int labeledExtract = SystemItems.getId(Extract.LABELED);
		int labeledExtractId1 = TestExtract.test_create(0, "Labeled #1", labeledExtract, cy3, false);
		int labeledExtractId2 = TestExtract.test_create(0, "Labeled #2", labeledExtract, cy5, false);
		int id4 = test_create(0, "Hyb #4", SystemItems.getId(PhysicalBioAssay.HYBRIDIZATION), 1, labeledExtractId1, labeledExtractId2);
		
		test_list(-1);
		
		TestExtract.test_list_bioassays(labeledExtractId1, 1);
		ok = ok && TestExtract.ok;

		// Standard test: Delete
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		test_delete(id);
		test_delete(id2);
		test_delete(id3);
		test_delete(id4);
		
		TestExtract.test_delete(labeledExtractId1);
		TestExtract.test_delete(labeledExtractId2);
		
		TestArraySlide.test_delete(arraySlideId);
		TestArrayBatch.test_delete(arrayBatchId);
		TestArrayDesign.test_delete(arrayDesignId);
		write("++Testing physical bioassays "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(int arraySlideId, String name, int subtypeId, int size, int... extractIds)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.PHYSICALBIOASSAY)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			PhysicalBioAssay h = PhysicalBioAssay.getNew(dc);
			h.setName(name);
			h.setSize(size);
			h.setDescription("Added at "+new Date());
			if (subtypeId != 0)
			{
				h.setItemSubtype(ItemSubtype.getById(dc, subtypeId));
			}
			if (arraySlideId != 0)
			{
				h.setArraySlide(ArraySlide.getById(dc, arraySlideId));
			}
			BioMaterialEvent evt = h.getCreationEvent();
			for (int extractId : extractIds)
			{
				Extract le = Extract.getById(dc, extractId);
				BioMaterialEventSource eventSource = evt.addSource(le);
				eventSource.setUsedQuantity(100.0f);
			}
			dc.saveItem(h);
			dc.commit();
			id = h.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(h, false);
			dc.reattachItem(h.getCreationEvent(), false);
			write_item(0, h);
			write("--Create physical bioassay OK");
		}
		catch (Throwable ex)
		{
			write("--Create physical bioassay FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			PhysicalBioAssay h = PhysicalBioAssay.getById(dc, id);
			write_item(0, h);
			write("--Load physical bioassay OK");
		}
		catch (Throwable ex)
		{
			write("--Load physical bioassay FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static boolean test_list(int expectedResults)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<PhysicalBioAssay> l = PhysicalBioAssay.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List physical bioassay OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List physical bioassay FAILED");
			ex.printStackTrace();
			ok = false;
			return false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return true;
	}

	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			PhysicalBioAssay h = PhysicalBioAssay.getById(dc, id);
			dc.deleteItem(h);
			Set<ItemProxy> using = h.getUsingItems();
			if (using.size() > 0) 
			{
			   throw new BaseException(h + " is used by " + using);
			}
			dc.commit();
			write("--Delete physical bioassay OK");
		}
		catch (Throwable ex)
		{
			write("--Delete physical bioassay FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_delete_all()
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<PhysicalBioAssay> l = PhysicalBioAssay.getQuery().list(dc);
			int numDeleted = Trashcan.delete(dc.getSessionControl(), l, true, null);
			if (numDeleted != l.size()) throw new BaseException("Could not delete all items: " + numDeleted + " of " + l.size());
			dc.commit();
			write("--Delete all physical bioassay OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--Delete all physical bioassay FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	
	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tType\tDescription\tArray slide\tLabeled extracts");
			write("-- \t-- \t--------- \t----\t-----------\t-----------\t----------------");
		}
	}
	static void write_item(int i, PhysicalBioAssay h)
		throws BaseException
	{
		if (!TestUtil.getSilent()) 
		{
			StringBuilder sb = new StringBuilder();
			sb.append("[");
			for (BioMaterial le : h.getCreationEvent().getSources().list(h.getDbControl()))
			{
				sb.append(le.toString());
				sb.append(",");
			}
			sb.append("]");
			System.out.println(i+":\t"+h.getId()+"\t"+h.getName()+"\t"+h.getItemSubtype()+"\t"+h.getDescription()+
				"\t"+h.getArraySlide()+"\t"+sb);
		}
	}
	static void write(String message)
	{
		System.out.println(message);
	}
	
	static void test_add_extract(int hybId, int extractId, int position, Float usedQuantity)
	{
		if (hybId == 0 || extractId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			PhysicalBioAssay h = PhysicalBioAssay.getById(dc, hybId);
			Extract le = Extract.getById(dc, extractId);
			BioMaterialEvent event = h.getCreationEvent();
			BioMaterialEventSource eventSource = event.addSource(le);
			eventSource.setUsedQuantity(usedQuantity);
			eventSource.setPosition(position);
			dc.commit();
			write("--Add extract to bioassay OK");
		}
		catch (Throwable ex)
		{
			write("--Add extract to bioassay FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
}
