/*
	$Id: TestGenericOverview.java 6100 2012-09-04 08:43:21Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.
	
	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Nameable;
import net.sf.basedb.core.SyncFilter.SourceItemTransform;
import net.sf.basedb.util.listable.ListableUtil;
import net.sf.basedb.util.listable.SourceItemTransformer;
import net.sf.basedb.util.listable.TransformContext;

public class TestSourceItemTransformer
{
	static boolean ok = true;

	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing source item transformer");
		
		// Create extracts: RNA, Library, Pooled library
		int rna1 = TestExtract.test_create(0, "RNA.1", 0, 0, false);
		int rna2 = TestExtract.test_create(0, "RNA.2", 0, 0, false);
		int lib1 = TestExtract.test_create(rna1, "Lib.1", 0, 0, false);
		int lib2 = TestExtract.test_create(rna2, "Lib.2", 0, 0, false);
		int pool = TestExtract.test_create_pooled("Pool.1", lib1, lib2);
		
		// Create physical bioassay: Flow cell
		int fc1 = TestPhysicalBioAssay.test_create(0, "FlowCell.1", 0, 1);
		TestPhysicalBioAssay.test_add_extract(fc1, pool, 1, null);
		
		// Create derived bioassay: Sequencing, Alignment
		int seqrun1 = TestDerivedBioAssay.test_create_root("SequencingRun.1", fc1, 0, 0, 0, 0, 0);
		
		int aligned1 = TestDerivedBioAssay.test_create_child("Aligned.1", lib1, 0, 0, 0, 0, seqrun1);
		int aligned2 = TestDerivedBioAssay.test_create_child("Aligned.2", lib2, 0, 0, 0, 0, seqrun1);
		
		SourceItemTransformer tDown = create_transformer(Item.EXTRACT, Item.DERIVEDBIOASSAY, SourceItemTransform.PARENT_TO_CHILD);
		SourceItemTransformer tUp = create_transformer(Item.DERIVEDBIOASSAY, Item.EXTRACT, SourceItemTransform.CHILD_TO_PARENT);
		
		// If we start at RNA level, we should get the Sequencing and only the Alignment that is linked
		test_transform(tDown, new Integer[] { rna1 }, new Integer[] { seqrun1, aligned1 });
		test_transform(tDown, new Integer[] { rna2 }, new Integer[] { seqrun1, aligned2 });
		// If we start at the pool level, we should get all alignments
		test_transform(tDown, new Integer[] { pool }, new Integer[] { seqrun1, aligned1, aligned2 });

		// If we start at the Alignment level, we should get the pool and only the Library and RNA that is linked
		test_transform(tUp, new Integer[] { aligned1 }, new Integer[] { pool, lib1, rna1 });
		test_transform(tUp, new Integer[] { aligned2 }, new Integer[] { pool, lib2, rna2 });
		// If we start at the Sequencing level, we should get all extracts
		test_transform(tUp, new Integer[] { seqrun1 }, new Integer[] { pool, lib1, lib2, rna1, rna2 });
		
		// Delete items
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();

		TestDerivedBioAssay.test_delete(aligned1);
		TestDerivedBioAssay.test_delete(aligned2);
		TestDerivedBioAssay.test_delete(seqrun1);
		TestPhysicalBioAssay.test_delete(fc1);
		TestExtract.test_delete(pool);
		TestExtract.test_delete(lib1);
		TestExtract.test_delete(lib2);
		TestExtract.test_delete(rna1);
		TestExtract.test_delete(rna2);

		write("++Testing source item transformer "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}
	
	
	static void write(String message)
	{
		System.out.println(message);
	}
	
	static SourceItemTransformer create_transformer(Item sourceType, Item targetType, SourceItemTransform transform)
	{
		return ListableUtil.getTransformerFactory(targetType).create(sourceType, transform);
	}

	
	static void test_transform(SourceItemTransformer transformer, Integer[] source, Integer[] expectedResult)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			
			Set<Integer> srcSet = new HashSet<>(Arrays.asList(source));
			Set<Integer> expectedResultSet = new HashSet<>(Arrays.asList(expectedResult)); 
			Set<Integer> result = transformer.transform(new TransformContext(dc), srcSet);

			write_transform(dc, transformer, srcSet, result);
			
			if (!result.equals(expectedResultSet))
			{
				throw new BaseException("Expected "+expectedResultSet+" results, not "+result);
			}
			
			write("--Transform " + transformer.getSourceItemType().name() + " -> " + transformer.getTargetItemType().name() + " OK");
		}
		catch (Throwable ex)
		{
			write("--Transform " + transformer.getSourceItemType().name() + " -> " + transformer.getTargetItemType().name() + " FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		
	}
	

	static void write_transform(DbControl dc, SourceItemTransformer transformer, Set<Integer> src, Set<Integer> result)
		throws BaseException
	{
		if (TestUtil.getSilent()) return;
		
		StringBuilder sb = new StringBuilder();
		sb.append(":"+transformer.getSourceItemType().name()+"\n");
		for (Integer id : src)
		{
			Nameable item = transformer.getSourceItemType().getById(dc, id);
			sb.append("   " + item.getName()+ "\t" + item.getId() +"\n");
		}
		sb.append(":" + transformer.getTargetItemType().name()+"\n");
		for (Integer id : result)
		{
			Nameable item = transformer.getTargetItemType().getById(dc, id);
			sb.append("   " + item.getName()+"\t"+item.getId()+"\n");
		}
		System.out.print(sb.toString());
	}

	
}
