/**
	$Id: RootTest.java 5827 2011-10-26 10:51:52Z nicklas $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.test.performance;

import java.util.ArrayList;
import java.util.List;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Experiment;
import net.sf.basedb.core.Formula;
import net.sf.basedb.core.ItemNotFoundException;
import net.sf.basedb.core.ItemParameterType;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.RawBioAssay;
import net.sf.basedb.core.RawDataType;
import net.sf.basedb.core.StringParameterType;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Orders;
import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.test.PluginUtil;
import net.sf.basedb.test.TestUtil;
import net.sf.basedb.util.Values;

/**
	Create a root bioassayset.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2011-10-26 12:51:52 +0200 (Wed, 26 Oct 2011) $
*/
public class RootTest
{
	public static boolean ok = true;
	public static final String DEFAULT_CHILD_PREFIX = "Root";
	
	/**
		Run the tests in this class.
	*/
	public static boolean run(String user, String password, 
			String sourcePrefix, String childPrefix,
			int iterations, int numSourcesStart, int numSourcesEnd, boolean execute)
	{
		DbControl dc = null;
		TestUtil.write("++Entering root bioassayset test\n");
		try
		{
			TestUtil.login(user, password);

			dc = TestUtil.getDbControl();
			
			// Configuration options
			if (childPrefix == null) childPrefix = DEFAULT_CHILD_PREFIX;

			Experiment experiment = PrepareTest.getExperiment(dc);
			RawDataType rawDataType = experiment.getRawDataType();
			Formula formula = Util.findFormula(dc, rawDataType, 
					Formula.Type.INTENSITY_EXPRESSION, Config.getIntensityFormula(rawDataType.getId()));
				List<String> expressions = formula.getFormulas();
			TestUtil.write("---------------------------------------------\n");
			TestUtil.write("      formula = " + formula.getName()+"\n");
			TestUtil.write("        ch(1) = " + expressions.get(0) + "\n");
			TestUtil.write("        ch(2) = " + expressions.get(1) + "\n");
			TestUtil.write("Source prefix = " + sourcePrefix + "\n");
			TestUtil.write(" Child prefix = " + childPrefix + "\n");
			TestUtil.write("      Execute = " + execute + "\n");
			TestUtil.write("---------------------------------------------\n");
			
			// Load source raw bioassays
			ItemQuery<RawBioAssay> rawQuery = experiment.getRawBioAssays();
			if (sourcePrefix != null)
			{
				// Only use raw bioassay sets starting with 'sourcePrefix'
				rawQuery.restrict(Restrictions.like(
						Hql.property("name"), Expressions.string(sourcePrefix + "%")));
			}
			rawQuery.order(Orders.asc(Hql.property("name")));
			List<RawBioAssay> sources = new ArrayList<RawBioAssay>(rawQuery.list(dc));
			int numSources = sources.size();
			if (numSources == 0)
			{
				throw new ItemNotFoundException("RawBioAssay set[name like " + sourcePrefix + "]");
			}
			dc.close();
			
			// Run tests
			long minTime = Long.MAX_VALUE;
			long maxTime = 0;
			long totalTime = 0;
			int firstIndex = 0;
			for (int i = 1; i <= iterations; ++i)
			{
				int useSources = numSourcesStart;
				while (useSources <= numSourcesEnd && useSources <= sources.size())
				{
					String childName = childPrefix + "." + i + "#" + useSources;
					if (firstIndex + useSources >= numSources)
					{
						firstIndex = 0;
					}
					List<RawBioAssay> subSources = sources.subList(firstIndex, firstIndex + useSources);
					long time = createRootBioAssaySet(experiment, subSources, formula, childName, execute) / 1000;
					firstIndex += useSources;
					useSources *= 2;
					if (execute)
					{
						totalTime += time;
						if (time < minTime) minTime = time;
						if (time > maxTime) maxTime = time;
						TestUtil.write(childName + "\t" + Values.formatTime(time) + "\n");
					}
					else
					{
						TestUtil.write("--Job created for: " + childName + "\n");
					}
				}
			}
			
			if (execute)
			{
				TestUtil.write("---------------------------------------------\n");
				TestUtil.write("Max time: " + Values.formatTime(maxTime) + "\n");
				TestUtil.write("Min time: " + Values.formatTime(minTime) + "\n");
			}
			TestUtil.write("---------------------------------------------\n");
		
			TestUtil.logout();
		}
		catch (Throwable t)
		{
			ok = false;
			t.printStackTrace();
		}
		finally
		{
			if (dc != null) dc.close();
		}
		TestUtil.write("++Leaving root bioassayset test (" + (ok ? "OK" : "FAILED") + ")\n");
		return ok;
	}

	/**
		Create a root bioassay set.
	*/
	public static long createRootBioAssaySet(Experiment experiment, List<RawBioAssay> sources, 
		Formula formula, String name, boolean execute)
		throws Exception
	{
		if (execute) TestUtil.write("--Create root bioassay set: " + name + "\n");
		DbControl dc = TestUtil.getDbControl();
		try
		{
			PluginDefinition plugin = PluginDefinition.getByClassName(dc, "net.sf.basedb.plugins.IntensityCalculatorPlugin");
			Job job = Job.getNew(dc, plugin, null, experiment);
			job.setName("PerformanceTest->Intensity: " + name);
			job.setSendMessage(false);
			job.setRemoveJobWhenFinished(false);
			job.setParameterValue("name", new StringParameterType(255, null, true), 
				name);
			job.setParameterValue("experiment", new ItemParameterType<Experiment>(Experiment.class, null),
				experiment);
			job.setParameterValues("rawBioAssays", new ItemParameterType<RawBioAssay>(RawBioAssay.class, null, true, 0, null), 
				sources);
			job.setParameterValue("formula", new ItemParameterType<Formula>(Formula.class, null), 
				formula);
			dc.saveItem(job);
			dc.commit();
			long time = 0;
			if (execute)
			{
				time = System.currentTimeMillis();
				PluginUtil.executeJob(job);
				time = System.currentTimeMillis() - time;
			}
			return time;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	
}
