/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2021 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/
'use strict';

var Upload = function()
{
	var upload = {};
	var zipFormats = [];
	
	var filesAndFolders;
	var currentFolder;
	
	var files = [];
	var uploadRequest;
	var uploadAborted = false;
	
	var totalFiles = 0;
	var currentFile = 0;
	var uploadedFiles = 0;
	var skippedFiles = 0;
	
	var uploadStartTime;
	var totalBytesToUpload = 0;
	var bytesAlreadyUploaded = 0;
	var skippedBytes = 0;
	
	/**
		Initialize the page.
	*/
	upload.initPage = function()
	{
		zipFormats = Data.json('page-data', 'zip-formats');

		Buttons.addClickHandler('btnUpload', upload.beginFileUpload);
		Buttons.addClickHandler('close', App.closeWindow);
		Buttons.addClickHandler('btnAbort', upload.abortUpload);
		Buttons.addClickHandler('failClose', App.closeWindow);

		filesAndFolders = window.opener.FileManager.getFilesToUpload();
		upload.processFilesAndFolders();
	}
	
	/**
		Get the ID of the best matching compressed format
	*/
	upload.getCompressedFormat = function(filename)
	{
		var longestMatch = 0;
		var bestFormat = null;
		
		for (var formatNo = 0; formatNo < zipFormats.length; formatNo++)
		{
			var zf = zipFormats[formatNo];
			for (var extNo = 0; extNo < zf.extensions.length; extNo++)
			{
				var ext = '.' + zf.extensions[extNo];
				var index = filename.lastIndexOf(ext);
				if (index > 0 && (index + ext.length == filename.length))
				{
					if (ext.length > longestMatch)
					{
						longestMatch = ext.length;
						bestFormat = zf;
					}
				}
			}
		}
		return bestFormat;
	}
	
	var currentDirectoryReader = null;
	/*
		Process all selected items (files and folder) and
		put them in the 'files[]' upload queue.
	*/
	upload.processFilesAndFolders = function()
	{
		if (filesAndFolders.length == 0)
		{
			// Nothing more to process -- start upload
			upload.summarizePreProcessing();
			return;
		}
		
		// Get and remove the first entry from the process queue
		// The filesAndFolder should contain 'FileSystemFileEntry' 
		// and 'FileSystemDirectoryEntry' object
		var ff = filesAndFolders.splice(0, 1)[0];
		currentFolder = ff.subfolder || '';
		// Ignore files/directories inside subfolders starting with '.' unless
		// the top folder that was selected is also hidden
		var ignore = ff.name.charAt(0)=='.' && (currentFolder.length > 0 && currentFolder.charAt(0)!='.');
		if (!ignore)
		{
			if (ff.isFile)
			{
				// Get the file item as a 'File' object (its an async operation)
				ff.file(upload.gotFile);
				return;
			}
			else if (ff.isDirectory)
			{
				// Save the currentFolder for later use in 'upload.gotFolder'
				currentFolder += ff.name+'/';
				// Read files and subfolders (its an async operation)
				currentDirectoryReader = ff.createReader();
				currentDirectoryReader.readEntries(upload.gotFolder);
				return;
			}
		}
		
		// Process next entry
		upload.processFilesAndFolders();
	}
	
	// We have a File item that should be upload
	// Store and continue processing
	upload.gotFile = function(file)
	{
		file.subfolder = currentFolder;
		files[files.length] = file;
		upload.processFilesAndFolders();
	}
	
	// We have a list of more 'FileSystemFileEntry'/'FileSystemDirectoryEntry'
	// object. Add them to the processing queue and continue with the next element
	upload.gotFolder = function(moreFilesAndFolders)
	{
		if (moreFilesAndFolders.length == 0) 
		{
			currentDirectoryReader = null;
			upload.processFilesAndFolders();
			return;
		}
		for (var i = 0; i < moreFilesAndFolders.length; i++)
		{
			var item = moreFilesAndFolders[i];
			item.subfolder = currentFolder;
			filesAndFolders[filesAndFolders.length] = item;
		}
		// We need to repeat this call until we get an empty result, since
		// some browsers (eg. chrome/edge) return a subset for each call.
		currentDirectoryReader.readEntries(upload.gotFolder);
	}
	
	upload.summarizePreProcessing = function()
	{
		if (files.length > 0)
		{
			files.sort(upload.sortFilesByFolder);
			var html = '<table class="filelist" style="width: 100%;">';
			var lastSubfolder = '';
			var pad = '<td colspan="2" class="name">';
			var numFolders = 1;
			var numCompressedFiles = 0;
			for (var i = 0; i < files.length; i++)
			{
				var f = files[i];
				totalBytesToUpload += f.size;
				totalFiles++;
				if (upload.getCompressedFormat(f.name)) numCompressedFiles++;
				if (totalFiles <= 500)
				{
					if (f.subfolder != lastSubfolder)
					{
						html += '<tr class="folder"><td colspan="3">'+Strings.encodeTags(f.subfolder)+'</td></tr>';
						lastSubfolder = f.subfolder;
						pad = '<td class="pad"></td><td class="name">';
						numFolders++;
					}
					html += '<tr class="file">'+pad+Strings.encodeTags(f.name)+'</td><td class="size">'+Numbers.formatBytes(f.size)+'</td></tr>';
				}
			}
			if (totalFiles > 500)
			{
				html += '<tr><td colspan="3" class="morefiles">+'+(totalFiles-500)+' more files...</td></tr>';
			}
			if (numCompressedFiles > 0) Doc.show('unpackSection');
			var numFilesMsg = totalFiles==1?'1 file' : totalFiles+' files';
			document.title = 'Uploading '+numFilesMsg+' to '+Data.get('page-data', 'directory-path');
			Doc.element('mainTitle').innerHTML = document.title;
			Doc.element('progressTitle').innerHTML = document.title;
			Doc.element('totalBytes').innerHTML = Numbers.formatBytes(totalBytesToUpload);
			
			Doc.element('allFiles').innerHTML = html;
			Doc.removeClass('btnUpload', 'disabled');
		}
		else
		{
			upload.displayError({'message': 'There are no files to upload'});
		}
	}
	
	upload.sortFilesByFolder = function(f1, f2)
	{
		if (f1.subfolder == f2.subfolder) return f1.name < f2.name ? -1 : 1;
		return f1.subfolder < f2.subfolder ? -1 : 1;
	}
	
	// Start the file uploading
	upload.beginFileUpload = function()
	{
		Doc.hide('preUploadForm');
		Doc.show('progressForm');
		uploadStartTime = new Date().getTime();
		upload.uploadNextFile();
		upload.sendProgressUpdateRequest();
	}
	
	var file;
	upload.uploadNextFile = function()
	{
		if (uploadAborted) return;		
		if (files.length == 0)
		{
			var numFilesMsg = uploadedFiles==1?'1 file' : uploadedFiles+' files';
			var msg = numFilesMsg+' upload sucessfully';
			if (skippedFiles > 0)
			{
				numFilesMsg = skippedFiles==1?'1 file' : skippedFiles+' files';
				msg += ' ('+numFilesMsg+' skipped)';
			}
			var url = '../../common/close_popup.jsp?refresh_opener=1&message='+encodeURIComponent(msg);
			window.top.location = url;
			return;
		}
		
		file = files.splice(0, 1)[0]; // Get first file and remove it from array
		currentFile++;

		var timeInSeconds = Math.round((new Date().getTime()-uploadStartTime)/1000);
		upload.displayProgress(bytesAlreadyUploaded, totalBytesToUpload, timeInSeconds);
		Doc.element('fileCount').innerHTML = currentFile+' of '+totalFiles; 
		Doc.element('fileName').innerHTML = Strings.encodeTags(file.name)+' ('+Numbers.formatBytes(file.size)+')';
		Doc.element('path').innerHTML = Data.get('page-data', 'directory-path')+'/'+file.subfolder;

		var frm = document.forms['upload'];
		if (frm.overwrite.value == 'always')
		{
			upload.doUpload(file, true);
		}
		else
		{
			var path = Data.get('page-data', 'directory-path')+'/'+file.subfolder+file.name;
			var url = 'ajax.jsp?ID='+App.getSessionId();
			url += '&cmd=CheckExistingFile';
			url += '&path='+encodeURIComponent(path);
			
			var request = Ajax.getXmlHttpRequest();
			request.open("GET", url, true);
			Ajax.setReadyStateHandler(request, upload.fileCheckCallback);
			request.send(null);
		}
	}
	
	upload.fileCheckCallback = function(request)
	{
		var check = JSON.parse(request.responseText);
		var frm = document.forms['upload'];
		var overwrite = frm.overwrite.value;
		
		if (check.fileExists)
		{
			if (overwrite == 'skipOrReplaceSize')
			{
				overwrite = check.fileSize != file.size ? 'always' : 'never';
			}
			if (overwrite == 'never') 
			{
				skippedFiles++;
				skippedBytes += file.size;
				bytesAlreadyUploaded += file.size;
				upload.uploadNextFile();
				return;
			}
		}
		upload.doUpload(file, overwrite == 'always');
	}
	
	upload.doUpload = function(file, overwrite)
	{
		if (uploadAborted) return;		
		
		// File name
		var frm = document.forms['upload'];
	
		var upl = new FormData();
		upl.set('subfolder', file.subfolder);
		upl.set('autoSelect', totalFiles == 1 ? 1 : 0); // No automatic selection if we upload more than one file
		upl.set('replace_existing', overwrite);
		if (frm.unpackCompressed.checked)
		{
			var zipFormat = upload.getCompressedFormat(file.name);
			if (zipFormat)
			{
				upl.set('zip_format', zipFormat.id);
				upl.set('zip_unpack', 1);
				upl.set('zip_overwrite', overwrite);
				upl.set('zip_keep', 0);
			}
		}
		upl.set('description', frm.description.value);
		upl.set('the_file', file);
		
		var url = 'Upload?ID='+App.getSessionId();
		url += '&directory_id='+Data.get('page-data', 'directory-id');

		uploadRequest = Ajax.getXmlHttpRequest();
		Ajax.setReadyStateHandler(uploadRequest, upload.uploadComplete, upload.uploadComplete);
		uploadRequest.open('POST', url, true);
		uploadRequest.send(upl);
	}


	upload.uploadComplete = function(request)
	{
		// If we have called uploadRequest.abort()
		// we do not need error handling here
		if (request.status == 0) return;
		var result = {};
		try
		{
			var contentType = request.getResponseHeader('Content-Type');
			if (request.status != 200 || contentType.indexOf('json') == -1)
			{
				throw new Error(request.responseText);
			}
			result = JSON.parse(request.responseText);			
		}
		catch (err)
		{
			result.status = 'error';
			result.message = err;
		}
		if (result.status == 'ok')
		{
			bytesAlreadyUploaded += result.transferredBytes;
			uploadedFiles++;
			upload.uploadNextFile();
		}
		else
		{
			upload.displayError(result);
		}
	}

	upload.sendProgressUpdateRequest = function()
	{
		var request = Ajax.getXmlHttpRequest();
		var url = 'ajax.jsp?ID='+App.getSessionId();
		url += '&cmd=GetProgress&'+Math.random();
		request.open("GET", url, true);
		Ajax.setReadyStateHandler(request, upload.progressUpdateCallback);
		request.send(null);
	}
	
	upload.progressUpdateCallback = function(request)
	{
		var progress = JSON.parse(request.responseText);
		if (progress)
		{
			if (progress.status == 'ok')
			{
				if (progress.file)
				{
					var transferredBytes = bytesAlreadyUploaded + progress.transferredBytes;
					var timeInSeconds = Math.round((new Date().getTime()-uploadStartTime)/1000);
					var message = progress.message;
					upload.displayProgress(transferredBytes, totalBytesToUpload, timeInSeconds, message);
				}
			}
			else
			{
				upload.displayError(progress);
				if (uploadRequest) uploadRequest.abort(); // Important!! Otherwise the browser may try to re-send the request
				return;
			}
		}
		setTimeout(upload.sendProgressUpdateRequest, 1500);
	}

	upload.displayProgress = function(transferredBytes, totalBytes, transferTime, message)
	{
		// Percent done
		var percentDone = totalBytes == 0 ? 0 : Math.round(100 * transferredBytes / totalBytes);
		Doc.element('percentText').innerHTML = percentDone+'%';
		var doneElement = Doc.element('percentDone');
		doneElement.style.width = percentDone+'%';
		var remainElement = Doc.element('percentRemain');
		remainElement.style.width = (100-percentDone)+'%';
		if (percentDone == 100) Doc.hide('percentRemain');
		
		// Bytes
		Doc.element('transferredBytes').innerHTML = Numbers.formatBytes(transferredBytes);
		Doc.element('totalBytes').innerHTML = Numbers.formatBytes(totalBytes);
		if (skippedBytes > 0)
		{
			var numFilesMsg = skippedFiles==1?'1 file' : skippedFiles+' files';
			Doc.element('skippedBytes').innerHTML = ' (Skipped '+numFilesMsg+'; '+Numbers.formatBytes(skippedBytes)+')';
		}

		// Times and rate
		Doc.element('transferTime').innerHTML = Numbers.formatSeconds(transferTime);
		var transferRate = transferTime == 0 ? 0 : transferredBytes / transferTime;
		var fileRate = transferTime == 0 ? 0 : currentFile / transferTime;
		Doc.element('transferRate').innerHTML = '(' + Numbers.formatBytes(transferRate) + '/s)';
		if (transferRate > 0 && fileRate > 0)
		{
			var remainTime = Math.max(Math.ceil((totalBytes-transferredBytes)/transferRate), Math.ceil((totalFiles-currentFile)/fileRate));
			if (remainTime < 0) remainTime = 0;
			Doc.element('remainTime').innerHTML = Numbers.formatSeconds(remainTime) + ' (estimated)';
		}
		
		// Message
		if (message)
		{
			Doc.element('message').innerHTML = message.replace(/(.*):(.*)/, '<th>$1</th><td>$2</td>');
		}
		else
		{
			Doc.element('message').innerHTML = '';
		}
	}

	upload.displayError = function(result)
	{
		Doc.show('uploadFailed');
		Doc.hide('progressForm');
		Doc.hide('preUploadForm');
		Doc.element('failTitle').innerHTML = result.title || 'File upload failed';
		Doc.element('failMessage').innerHTML = result.message || result.stacktrace;
		if (result.status == 'abort') setTimeout(App.closeWindow, 1500);
	}

	upload.abortUpload = function()
	{
		if (confirm('Are you sure you want to abort the upload? It cannot be resumed.'))
		{
			/*
			Aborting is tricky. See tickets:
			http://base.thep.lu.se/ticket/2018
			http://base.thep.lu.se/ticket/2019
			
			Sending "Abort" command to the server AND 
			uploadRequest.abort() should tell both ends.
			The "abort" is catched by the progress updater that
			also display the error message.
			*/
			uploadAborted = true;
			if (uploadRequest) uploadRequest.abort();
			
			var url = 'ajax.jsp?ID='+App.getSessionId();
			url += '&cmd=Abort&'+Math.random();
			var request = Ajax.getXmlHttpRequest();
			Ajax.setReadyStateHandler(request, upload.abortProgressCallback);
			request.open("GET", url, true);
			request.send(null);
		}
	}

	upload.abortProgressCallback = function(request)
	{
		var progress = JSON.parse(request.responseText);
		if (!progress.aborted) 
		{
			// There was no active upload, so we simply abort...
			upload.displayError({'status': 'abort', 'message': 'The upload was aborted'});
		}
	}

	return upload;
}();

Doc.onLoad(Upload.initPage);
